from _typeshed import Incomplete, StrOrBytesPath
from collections.abc import Callable, Iterable, Sequence
from datetime import datetime
from typing import Any
from typing_extensions import TypeAlias

from cryptography.hazmat.primitives.asymmetric.dsa import DSAPrivateKey, DSAPublicKey
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
from cryptography.x509 import Certificate, CertificateRevocationList, CertificateSigningRequest

_Key: TypeAlias = DSAPrivateKey | DSAPublicKey | RSAPrivateKey | RSAPublicKey

FILETYPE_PEM: int
FILETYPE_ASN1: int
FILETYPE_TEXT: int

TYPE_RSA: int
TYPE_DSA: int
TYPE_DH: int
TYPE_EC: int

class _EllipticCurve:
    def __init__(self, lib: Incomplete | None, nid: int, name: str) -> None: ...

class Error(Exception): ...

class PKey:
    def __init__(self) -> None: ...
    def bits(self) -> int: ...
    def check(self) -> bool: ...
    @classmethod
    def from_cryptography_key(cls, crypto_key: _Key) -> PKey: ...
    def generate_key(self, type: int, bits: int) -> None: ...
    def to_cryptography_key(self) -> _Key: ...
    def type(self) -> int: ...

class X509Name:
    countryName: str
    C: str
    stateOrProvinceName: str
    ST: str
    localityName: str
    L: str
    organizationName: str
    O: str
    organizationalUnitName: str
    OU: str
    commonName: str
    CN: str
    emailAddress: str
    def __init__(self, name: X509Name) -> None: ...
    def der(self) -> bytes: ...
    def get_components(self) -> list[tuple[bytes, bytes]]: ...
    def hash(self) -> int: ...

class X509:
    def __init__(self) -> None: ...
    def add_extensions(self, extensions: Iterable[X509Extension]) -> None: ...
    def digest(self, digest_name: str) -> bytes: ...
    @classmethod
    def from_cryptography(cls, crypto_cert: Certificate) -> X509: ...
    def get_extension(self, index: int) -> X509Extension: ...
    def get_extension_count(self) -> int: ...
    def get_issuer(self) -> X509Name: ...
    def get_notAfter(self) -> bytes | None: ...
    def get_notBefore(self) -> bytes | None: ...
    def get_pubkey(self) -> PKey: ...
    def get_serial_number(self) -> int: ...
    def get_signature_algorithm(self) -> bytes: ...
    def get_subject(self) -> X509Name: ...
    def get_version(self) -> int: ...
    def gmtime_adj_notAfter(self, amount: int) -> None: ...
    def gmtime_adj_notBefore(self, amount: int) -> None: ...
    def has_expired(self) -> bool: ...
    def set_issuer(self, issuer: X509Name) -> None: ...
    def set_notAfter(self, when: bytes) -> None: ...
    def set_notBefore(self, when: bytes) -> None: ...
    def set_pubkey(self, pkey: PKey) -> None: ...
    def set_serial_number(self, serial: int) -> None: ...
    def set_subject(self, subject: X509Name) -> None: ...
    def set_version(self, version: int) -> None: ...
    def sign(self, pkey: PKey, digest: str) -> None: ...
    def subject_name_hash(self) -> bytes: ...
    def to_cryptography(self) -> Certificate: ...

class X509Req:
    def __init__(self) -> None: ...
    def add_extensions(self, extensions: Iterable[X509Extension]) -> None: ...
    @classmethod
    def from_cryptography(cls, crypto_req: CertificateSigningRequest) -> X509Req: ...
    def get_extensions(self) -> list[X509Extension]: ...
    def get_pubkey(self) -> PKey: ...
    def get_subject(self) -> X509Name: ...
    def get_version(self) -> int: ...
    def set_pubkey(self, pkey: PKey) -> None: ...
    def set_version(self, version: int) -> None: ...
    def sign(self, pkey: PKey, digest: str) -> None: ...
    def to_cryptography(self) -> CertificateSigningRequest: ...
    def verify(self, pkey: PKey) -> bool: ...

# deprecated
class X509Extension:
    def __init__(
        self, type_name: bytes, critical: bool, value: bytes, subject: X509 | None = None, issuer: X509 | None = None
    ) -> None: ...
    def get_critical(self) -> bool: ...
    def get_data(self) -> bytes: ...
    def get_short_name(self) -> bytes: ...

# deprecated
class Revoked:
    def __init__(self) -> None: ...
    def all_reasons(self) -> list[bytes]: ...
    def get_reason(self) -> bytes | None: ...
    def get_rev_date(self) -> bytes: ...
    def get_serial(self) -> bytes: ...
    def set_reason(self, reason: bytes | None) -> None: ...
    def set_rev_date(self, when: bytes) -> None: ...
    def set_serial(self, hex_str: bytes) -> None: ...

# deprecated
class CRL:
    def __init__(self) -> None: ...
    def add_revoked(self, revoked: Revoked) -> None: ...
    def export(self, cert: X509, key: PKey, type: int = 1, days: int = 100, digest: bytes = ...) -> bytes: ...
    @classmethod
    def from_cryptography(cls, crypto_crl: CertificateRevocationList) -> CRL: ...
    def get_issuer(self) -> X509Name: ...
    def get_revoked(self) -> tuple[Revoked, ...]: ...
    def set_lastUpdate(self, when: bytes) -> None: ...
    def set_nextUpdate(self, when: bytes) -> None: ...
    def set_version(self, version: int) -> None: ...
    def sign(self, issuer_cert: X509, issuer_key: PKey, digest: bytes) -> None: ...
    def to_cryptography(self) -> CertificateRevocationList: ...

class X509Store:
    def __init__(self) -> None: ...
    def add_cert(self, cert: X509) -> None: ...
    def add_crl(self, crl: CRL | CertificateRevocationList) -> None: ...
    def load_locations(self, cafile: StrOrBytesPath, capath: StrOrBytesPath | None = None) -> None: ...
    def set_flags(self, flags: int) -> None: ...
    def set_time(self, vfy_time: datetime) -> None: ...

class X509StoreContext:
    def __init__(self, store: X509Store, certificate: X509, chain: Sequence[X509] | None = None) -> None: ...
    def get_verified_chain(self) -> list[X509]: ...
    def set_store(self, store: X509Store) -> None: ...
    def verify_certificate(self) -> None: ...

class X509StoreContextError(Exception):
    errors: list[Any]
    certificate: X509
    def __init__(self, message: str, errors: list[Any], certificate: X509) -> None: ...

class X509StoreFlags:
    CRL_CHECK: int
    CRL_CHECK_ALL: int
    IGNORE_CRITICAL: int
    X509_STRICT: int
    ALLOW_PROXY_CERTS: int
    POLICY_CHECK: int
    EXPLICIT_POLICY: int
    INHIBIT_MAP: int
    NOTIFY_POLICY: int
    CHECK_SS_SIGNATURE: int
    CB_ISSUER_CHECK: int
    PARTIAL_CHAIN: int

# deprecated
class PKCS12:
    def __init__(self) -> None: ...
    def export(self, passphrase: bytes | None = None, iter: int = 2048, maciter: int = 1) -> bytes: ...
    def get_ca_certificates(self) -> tuple[X509, ...]: ...
    def get_certificate(self) -> X509: ...
    def get_friendlyname(self) -> bytes | None: ...
    def get_privatekey(self) -> PKey: ...
    def set_ca_certificates(self, cacerts: Iterable[X509] | None) -> None: ...
    def set_certificate(self, cert: X509) -> None: ...
    def set_friendlyname(self, name: bytes | None) -> None: ...
    def set_privatekey(self, pkey: PKey) -> None: ...

# deprecated
class NetscapeSPKI:
    def __init__(self) -> None: ...
    def b64_encode(self) -> bytes: ...
    def get_pubkey(self) -> PKey: ...
    def set_pubkey(self, pkey: PKey) -> None: ...
    def sign(self, pkey: PKey, digest: str) -> None: ...
    def verify(self, key: PKey) -> bool: ...

def get_elliptic_curves() -> set[_EllipticCurve]: ...
def get_elliptic_curve(name: str) -> _EllipticCurve: ...
def dump_certificate(type: int, cert: X509) -> bytes: ...
def load_certificate(type: int, buffer: bytes) -> X509: ...
def dump_certificate_request(type: int, req: X509Req) -> bytes: ...
def load_certificate_request(type: int, buffer: bytes) -> X509Req: ...
def dump_privatekey(
    type: int, pkey: PKey, cipher: str | None = None, passphrase: bytes | Callable[[], bytes] | None = None
) -> bytes: ...
def load_privatekey(type: int, buffer: str | bytes, passphrase: bytes | Callable[[], bytes] | None = None) -> PKey: ...
def dump_publickey(type: int, pkey: PKey) -> bytes: ...
def load_publickey(type: int, buffer: str | bytes) -> PKey: ...
def dump_crl(type: int, crl: CRL) -> bytes: ...  # deprecated
def load_crl(type: int, buffer: str | bytes) -> CRL: ...  # deprecated
def sign(pkey: PKey, data: str | bytes, digest: str) -> bytes: ...  # deprecated
def verify(cert: X509, signature: bytes, data: str | bytes, digest: str) -> None: ...  # deprecated



from _typeshed import Incomplete
from typing import Any, TypeVar

from pygments.formatter import Formatter

_T = TypeVar("_T", str, bytes)

class EscapeSequence:
    fg: Any
    bg: Any
    bold: Any
    underline: Any
    italic: Any
    def __init__(
        self,
        fg: Incomplete | None = None,
        bg: Incomplete | None = None,
        bold: bool = False,
        underline: bool = False,
        italic: bool = False,
    ) -> None: ...
    def escape(self, attrs): ...
    def color_string(self): ...
    def true_color_string(self): ...
    def reset_string(self): ...

class Terminal256Formatter(Formatter[_T]):
    name: str
    aliases: Any
    filenames: Any
    xterm_colors: Any
    best_match: Any
    style_string: Any
    usebold: Any
    useunderline: Any
    useitalic: Any
    linenos: Any
    def format(self, tokensource, outfile): ...
    def format_unencoded(self, tokensource, outfile) -> None: ...

class TerminalTrueColorFormatter(Terminal256Formatter[_T]):
    name: str
    aliases: Any
    filenames: Any



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def current_flow_closeness_centrality(G, weight: Incomplete | None = None, dtype=..., solver: str = "lu"): ...

information_centrality = current_flow_closeness_centrality



from collections.abc import Sequence
from re import Pattern
from typing import Final

from Xlib._typing import ErrorHandler
from Xlib.protocol import request, rq
from Xlib.xobject import resource

rgb_res: Final[list[Pattern[str]]]

class Colormap(resource.Resource):
    __colormap__ = resource.Resource.__resource__
    def free(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def copy_colormap_and_free(self, scr_cmap: int) -> Colormap: ...
    def install_colormap(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def uninstall_colormap(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def alloc_color(self, red: int, green: int, blue: int) -> request.AllocColor: ...
    def alloc_named_color(self, name: str) -> request.AllocColor | request.AllocNamedColor | None: ...
    def alloc_color_cells(self, contiguous: bool, colors: int, planes: int) -> request.AllocColorCells: ...
    def alloc_color_planes(self, contiguous: bool, colors: int, red: int, green: int, blue: int) -> request.AllocColorPlanes: ...
    def free_colors(self, pixels: Sequence[int], plane_mask: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def store_colors(self, items: dict[str, int], onerror: ErrorHandler[object] | None = None) -> None: ...
    def store_named_color(self, name: str, pixel: int, flags: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def query_colors(self, pixels: Sequence[int]) -> rq.Struct: ...
    def lookup_color(self, name: str) -> request.LookupColor: ...



from _typeshed import OptExcInfo, StrOrBytesPath, SupportsWrite
from _typeshed.wsgi import WSGIApplication, WSGIEnvironment
from collections.abc import Callable, Container, Iterable, Iterator
from http.client import HTTPMessage
from io import BufferedIOBase, BufferedReader
from logging import Logger
from types import TracebackType
from typing import Any, ClassVar, Literal, Protocol, TypeVar, overload
from typing_extensions import Self

from gevent.baseserver import _Spawner
from gevent.server import StreamServer
from gevent.socket import socket as _GeventSocket
from gevent.ssl import SSLContext

__all__ = ["WSGIServer", "WSGIHandler", "LoggingLogAdapter", "Environ", "SecureEnviron", "WSGISecureEnviron"]

_T = TypeVar("_T")

class _LogOutputStream(SupportsWrite[str], Protocol):
    def writelines(self, lines: Iterable[str], /) -> None: ...
    def flush(self) -> None: ...

class Input:
    rfile: BufferedReader
    content_length: int | None
    socket: _GeventSocket | None
    position: int
    chunked_input: bool
    chunk_length: int
    def __init__(
        self, rfile: BufferedReader, content_length: int | None, socket: _GeventSocket | None = None, chunked_input: bool = False
    ) -> None: ...
    def read(self, length: int | None = None) -> bytes: ...
    def readline(self, size: int | None = None) -> bytes: ...
    def readlines(self, hint: object | None = None) -> list[bytes]: ...
    def __iter__(self) -> Self: ...
    def next(self) -> bytes: ...
    __next__ = next

class OldMessage(HTTPMessage):
    status: str
    def __init__(self) -> None: ...
    @overload
    def getheader(self, name: str, default: None = None) -> str | None: ...
    @overload
    def getheader(self, name: str, default: _T) -> str | _T: ...
    @property
    def headers(self) -> Iterator[str]: ...
    @property
    def typeheader(self) -> str | None: ...

class WSGIHandler:
    protocol_version: str
    def MessageClass(self, fp: BufferedIOBase) -> OldMessage: ...
    status: str | None
    response_headers: list[tuple[str, str]] | None
    code: int | None
    provided_date: str | None
    provided_content_length: str | None
    close_connection: bool
    time_start: float
    time_finish: float
    headers_sent: bool
    response_use_chunked: bool
    connection_upgraded: bool
    environ: WSGIEnvironment | None
    application: WSGIApplication | None
    requestline: str | None
    response_length: int
    result: Iterable[bytes] | None
    wsgi_input: Input | None
    content_length: int
    headers: OldMessage
    request_version: str | None
    command: str | None
    path: str | None
    socket: _GeventSocket
    client_address: str
    server: WSGIServer
    rfile: BufferedReader
    def __init__(self, sock: _GeventSocket, address: str, server: WSGIServer) -> None: ...
    def handle(self) -> None: ...
    def read_request(self, raw_requestline: str) -> OldMessage: ...
    def log_error(self, msg: str, *args: object) -> None: ...
    def read_requestline(self) -> str: ...
    def handle_one_request(self) -> tuple[str, bytes] | Literal[True] | None: ...
    def finalize_headers(self) -> None: ...
    ApplicationError: type[AssertionError]
    def write(self, data: bytes) -> None: ...
    def start_response(
        self, status: str, headers: list[tuple[str, str]], exc_info: OptExcInfo | None = None
    ) -> Callable[[bytes], None]: ...
    def log_request(self) -> None: ...
    def format_request(self) -> str: ...
    def process_result(self) -> None: ...
    def run_application(self) -> None: ...
    ignored_socket_errors: tuple[int, ...]
    def handle_one_response(self) -> None: ...
    def handle_error(self, t: type[BaseException] | None, v: BaseException | None, tb: TracebackType | None) -> None: ...
    def get_environ(self) -> WSGIEnvironment: ...

class LoggingLogAdapter:
    def __init__(self, logger: Logger, level: int = 20) -> None: ...
    def write(self, msg: str) -> None: ...
    def flush(self) -> None: ...
    def writelines(self, lines: Iterable[str]) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: object) -> None: ...
    def __delattr__(self, name: str) -> None: ...

class Environ(WSGIEnvironment): ...

class SecureEnviron(Environ):
    default_secure_repr: ClassVar[bool]
    default_whitelist_keys: ClassVar[Container[str]]
    default_print_masked_keys: ClassVar[bool]
    secure_repr: bool
    whitelist_keys: Container[str]
    print_masked_keys: bool

class WSGISecureEnviron(SecureEnviron): ...

class WSGIServer(StreamServer):
    handler_class: type[WSGIHandler]
    log: _LogOutputStream
    error_log: _LogOutputStream
    environ_class: type[WSGIEnvironment]
    secure_environ_class: type[SecureEnviron]
    base_env: WSGIEnvironment
    application: WSGIApplication
    @overload
    def __init__(
        self,
        listener: _GeventSocket | tuple[str, int] | str,
        application: WSGIApplication | None = None,
        backlog: int | None = None,
        spawn: _Spawner = "default",
        log: str | Logger | _LogOutputStream | None = "default",
        error_log: str | Logger | _LogOutputStream | None = "default",
        handler_class: type[WSGIHandler] | None = None,
        environ: WSGIEnvironment | None = None,
        *,
        ssl_context: SSLContext,
        server_side: bool = True,
        do_handshake_on_connect: bool = True,
        suppress_ragged_eofs: bool = True,
    ) -> None: ...
    @overload
    def __init__(
        self,
        listener: _GeventSocket | tuple[str, int] | str,
        application: WSGIApplication | None = None,
        backlog: int | None = None,
        spawn: _Spawner = "default",
        log: str | Logger | _LogOutputStream | None = "default",
        error_log: str | Logger | _LogOutputStream | None = "default",
        handler_class: type[WSGIHandler] | None = None,
        environ: WSGIEnvironment | None = None,
        *,
        keyfile: StrOrBytesPath = ...,
        certfile: StrOrBytesPath = ...,
        server_side: bool = True,
        cert_reqs: int = ...,
        ssl_version: int = ...,
        ca_certs: str = ...,
        do_handshake_on_connect: bool = True,
        suppress_ragged_eofs: bool = True,
        ciphers: str = ...,
    ) -> None: ...
    environ: WSGIEnvironment
    def set_environ(self, environ: WSGIEnvironment | None = None) -> None: ...
    max_accept: int
    def set_max_accept(self) -> None: ...
    def get_environ(self) -> WSGIEnvironment: ...
    def init_socket(self) -> None: ...
    def update_environ(self) -> None: ...
    def handle(self, sock: _GeventSocket, address: str) -> None: ...



from collections.abc import Iterator, Mapping
from typing import Any
from typing_extensions import Self
from xml.sax import handler, xmlreader

def is_string(x: object) -> bool: ...

class Element:
    children: list[Element]
    is_root: bool
    cdata: str
    def __init__(self, name: str | None, attributes: Mapping[str, Any] | None) -> None: ...
    def add_child(self, element: Element) -> None: ...
    def add_cdata(self, cdata: str) -> None: ...
    def get_attribute(self, key: str) -> Any | None: ...
    def get_elements(self, name: str | None = ...) -> list[Element]: ...
    def __getitem__(self, key: str) -> Any | None: ...
    def __getattr__(self, key: str) -> Element: ...
    def __hasattribute__(self, name: str) -> bool: ...
    def __iter__(self) -> Iterator[Self]: ...
    def __bool__(self) -> bool: ...
    __nonzero__ = __bool__
    def __eq__(self, val: object) -> bool: ...
    def __dir__(self) -> list[str]: ...
    def __len__(self) -> int: ...
    def __contains__(self, key: str) -> bool: ...

class Handler(handler.ContentHandler):
    root: Element
    elements: list[Element]
    def __init__(self) -> None: ...
    def startElement(self, name: str, attributes: xmlreader.AttributesImpl) -> None: ...
    def endElement(self, name: str) -> None: ...
    def characters(self, cdata: str) -> None: ...

def parse(filename: str, **parser_features: bool) -> Element: ...
def is_url(string: str) -> bool: ...



from ...extend.operation import ExtendedOperation
from ...protocol.novell import Identity

class GetBindDn(ExtendedOperation):
    request_name: str
    response_name: str
    response_attribute: str
    asn1_spec: Identity
    def config(self) -> None: ...
    def populate_result(self) -> None: ...



from typing import Any

from waitress.server import create_server as create_server

def serve(app: Any, **kw: Any) -> None: ...
def serve_paste(app: Any, global_conf: Any, **kw: Any) -> int: ...
def profile(cmd: Any, globals: Any, locals: Any, sort_order: tuple[str, ...], callers: bool) -> None: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def current_flow_betweenness_centrality_subset(
    G, sources, targets, normalized: bool = True, weight: Incomplete | None = None, dtype=..., solver: str = "lu"
): ...
@_dispatch
def edge_current_flow_betweenness_centrality_subset(
    G, sources, targets, normalized: bool = True, weight: Incomplete | None = None, dtype=..., solver: str = "lu"
): ...



from collections.abc import Callable, Iterable, Mapping
from typing import Any, NoReturn, TypedDict, TypeVar
from typing_extensions import NotRequired

class _DeprecateProperty(TypedDict):
    to_be_removed_in_version: str
    client_property: str
    new_property: NotRequired[str]

_T = TypeVar("_T")
_K = TypeVar("_K")
_V = TypeVar("_V")

def raise_for_error(
    method: str,
    url: str,
    status_code: int,
    message: str | None = None,
    errors: Iterable[Any] | str | None = None,
    text: str | None = None,
    json: object | None = None,
) -> NoReturn: ...
def aliased_parameter(
    name: str, *aliases: str, removed_in_version: str | None, position: int | None = None, raise_on_multiple: bool = True
) -> Callable[..., Any]: ...
def generate_parameter_deprecation_message(
    to_be_removed_in_version: str, old_parameter_name: str, new_parameter_name: str | None = None, extra_notes: str | None = None
) -> str: ...
def generate_method_deprecation_message(
    to_be_removed_in_version: str, old_method_name: str, method_name: str | None = None, module_name: str | None = None
) -> str: ...
def generate_property_deprecation_message(
    to_be_removed_in_version: str, old_name: str, new_name: str, new_attribute: str, module_name: str = "Client"
) -> str: ...
def getattr_with_deprecated_properties(obj: object, item: str, deprecated_properties: dict[str, _DeprecateProperty]) -> Any: ...
def deprecated_method(to_be_removed_in_version: str, new_method: Callable[..., Any] | None = None) -> Callable[..., Any]: ...
def validate_list_of_strings_param(param_name: str, param_argument: Iterable[Any] | str) -> None: ...
def list_to_comma_delimited(list_param: Iterable[str] | None) -> str: ...
def get_token_from_env() -> str | None: ...
def comma_delimited_to_list(list_param: Iterable[_T]) -> Iterable[_T]: ...

# the docstring states that this function returns a bool, but the code does not return anything
def validate_pem_format(param_name: str, param_argument: str) -> None: ...
def remove_nones(params: Mapping[_K, _V | None]) -> Mapping[_K, _V]: ...
def format_url(format_str: str, *args: Any, **kwargs: Any) -> str: ...



import sys
from collections.abc import Generator, Hashable

from networkx.classes.graph import Graph, _Node
from networkx.exception import NetworkXException
from networkx.utils.backends import _dispatch

class NetworkXTreewidthBoundExceeded(NetworkXException): ...

@_dispatch
def is_chordal(G: Graph[Hashable]) -> bool: ...
@_dispatch
def find_induced_nodes(G: Graph[_Node], s: _Node, t: _Node, treewidth_bound: float = sys.maxsize) -> set[_Node]: ...
@_dispatch
def chordal_graph_cliques(G: Graph[_Node]) -> Generator[frozenset[_Node], None, None]: ...
@_dispatch
def chordal_graph_treewidth(G: Graph[Hashable]) -> int: ...



from collections.abc import Iterable
from typing import Any, AnyStr, Final, Literal, NamedTuple
from typing_extensions import TypeAlias

_Unit: TypeAlias = Literal["pt", "mm", "cm", "in"]

PIL_MEM_BLOCK_SIZE_IN_MIB: Final = 16

class Padding(NamedTuple):
    top: float = 0
    right: float = 0
    bottom: float = 0
    left: float = 0
    @classmethod
    def new(cls, padding: float | tuple[float, ...] | list[float]): ...

def buffer_subst(buffer: bytearray, placeholder: str, value: str) -> bytearray: ...
def escape_parens(s: AnyStr) -> AnyStr: ...
def get_scale_factor(unit: _Unit | float) -> float: ...
def convert_unit(
    # to_convert has a recursive type
    to_convert: float | Iterable[float | Iterable[Any]],
    old_unit: str | float,
    new_unit: str | float,
) -> float | tuple[float, ...]: ...
def print_mem_usage(prefix: str) -> None: ...
def get_mem_usage(prefix: str) -> str: ...
def get_process_rss() -> str: ...
def get_process_rss_as_mib() -> float | None: ...
def get_process_heap_and_stack_sizes() -> tuple[str, str]: ...
def get_pymalloc_allocated_over_total_size() -> str: ...
def get_gc_managed_objs_total_size() -> str: ...
def get_tracemalloc_traced_memory() -> str: ...
def get_pillow_allocated_memory() -> str: ...



from _typeshed.wsgi import WSGIEnvironment
from collections.abc import Iterator, MutableMapping

from webob.multidict import MultiDict

class ResponseHeaders(MultiDict[str, str]): ...

class EnvironHeaders(MutableMapping[str, str]):
    environ: WSGIEnvironment
    def __init__(self, environ: WSGIEnvironment) -> None: ...
    def __getitem__(self, hname: str) -> str: ...
    def __setitem__(self, hname: str, value: str) -> None: ...
    def __delitem__(self, hname: str) -> None: ...
    def keys(self) -> list[str]: ...  # type: ignore[override]
    def __contains__(self, hname: object) -> bool: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[str]: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def pagerank(
    G,
    alpha: float = 0.85,
    personalization: Incomplete | None = None,
    max_iter: int = 100,
    tol: float = 1e-06,
    nstart: Incomplete | None = None,
    weight: str = "weight",
    dangling: Incomplete | None = None,
): ...
@_dispatch
def google_matrix(
    G,
    alpha: float = 0.85,
    personalization: Incomplete | None = None,
    nodelist: Incomplete | None = None,
    weight: str = "weight",
    dangling: Incomplete | None = None,
): ...



from decimal import Decimal
from typing import Final

from braintree.merchant_account import MerchantAccount
from braintree.resource import Resource

class Disbursement(Resource):
    class Type:
        Credit: Final = "credit"
        Debit: Final = "debit"

    amount: Decimal
    merchant_account: MerchantAccount
    def __init__(self, gateway, attributes) -> None: ...
    def transactions(self): ...
    def is_credit(self): ...
    def is_debit(self): ...



from typing import Any, Literal
from typing_extensions import Self

from stripe.api_resources.abstract import CreateableAPIResource, DeletableAPIResource, ListableAPIResource

class TestClock(CreateableAPIResource, DeletableAPIResource, ListableAPIResource):
    OBJECT_NAME: Literal["test_helpers.test_clock"]

    @classmethod
    def advance(cls, idempotency_key: str | None = None, **params: Any) -> Self: ...



from _typeshed import Incomplete

class UsBankAccountVerificationGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def confirm_micro_transfer_amounts(self, verification_id, amounts): ...
    def find(self, verification_id): ...
    def search(self, *query): ...



from _typeshed import Incomplete
from typing import Final

from braintree.resource import Resource

class DocumentUpload(Resource):
    class Kind:
        EvidenceDocument: Final = "evidence_document"

    @staticmethod
    def create(params: Incomplete | None = None): ...
    @staticmethod
    def create_signature(): ...
    def __init__(self, gateway, attributes) -> None: ...



from networkx.algorithms.coloring.equitable_coloring import equitable_color as equitable_color
from networkx.algorithms.coloring.greedy_coloring import *

__all__ = ["greedy_color", "equitable_color"]



import io
import types
from _typeshed import WriteableBuffer
from contextlib import AbstractContextManager

from ._pygit2 import Blob, Oid
from .enums import BlobFilter

class _BlobIO(io.RawIOBase):
    def __init__(self, blob: Blob, as_path: str | None = None, flags: BlobFilter = ..., commit_id: Oid | None = None) -> None: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: types.TracebackType | None
    ) -> None: ...
    def isatty() -> bool: ...  # type: ignore[misc]
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def seekable(self) -> bool: ...
    def readinto(self, b: WriteableBuffer, /) -> int: ...
    def close(self) -> None: ...

class BlobIO(io.BufferedReader, AbstractContextManager[_BlobIO]):  # type: ignore[misc]
    def __init__(self, blob: Blob, as_path: str | None = None, flags: BlobFilter = ..., commit_id: Oid | None = None) -> None: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: types.TracebackType | None
    ) -> None: ...



import abc
from _typeshed import Incomplete
from decimal import Decimal
from typing import NamedTuple

from qrcode.image.styles.moduledrawers.base import QRModuleDrawer
from qrcode.image.svg import SvgFragmentImage, SvgPathImage

ANTIALIASING_FACTOR: int

class Coords(NamedTuple):
    x0: Decimal
    y0: Decimal
    x1: Decimal
    y1: Decimal
    xh: Decimal
    yh: Decimal

class BaseSvgQRModuleDrawer(QRModuleDrawer, metaclass=abc.ABCMeta):
    img: SvgFragmentImage
    size_ratio: Incomplete
    def __init__(self, *, size_ratio: Decimal = ..., **kwargs) -> None: ...
    box_delta: Incomplete
    box_size: Incomplete
    box_half: Incomplete
    def initialize(self, *args, **kwargs) -> None: ...
    def coords(self, box) -> Coords: ...

class SvgQRModuleDrawer(BaseSvgQRModuleDrawer, metaclass=abc.ABCMeta):
    tag: str
    tag_qname: Incomplete
    def initialize(self, *args, **kwargs) -> None: ...
    def drawrect(self, box, is_active: bool): ...
    @abc.abstractmethod
    def el(self, box): ...

class SvgSquareDrawer(SvgQRModuleDrawer):
    unit_size: Incomplete
    def initialize(self, *args, **kwargs) -> None: ...
    def el(self, box): ...

class SvgCircleDrawer(SvgQRModuleDrawer):
    tag: str
    radius: Incomplete
    def initialize(self, *args, **kwargs) -> None: ...
    def el(self, box): ...

class SvgPathQRModuleDrawer(BaseSvgQRModuleDrawer, metaclass=abc.ABCMeta):
    img: SvgPathImage
    def drawrect(self, box, is_active: bool): ...
    @abc.abstractmethod
    def subpath(self, box) -> str: ...

class SvgPathSquareDrawer(SvgPathQRModuleDrawer):
    def subpath(self, box) -> str: ...

class SvgPathCircleDrawer(SvgPathQRModuleDrawer):
    def initialize(self, *args, **kwargs) -> None: ...
    def subpath(self, box) -> str: ...



from _typeshed import Incomplete

def pid_exists(pid): ...
def wait_pid(
    pid,
    timeout: Incomplete | None = None,
    proc_name: Incomplete | None = None,
    _waitpid=...,
    _timer=...,
    _min=...,
    _sleep=...,
    _pid_exists=...,
): ...
def disk_usage(path): ...
def get_terminal_map(): ...



from collections.abc import Iterable, Mapping, Sequence
from typing import Any, Literal
from typing_extensions import TypeAlias

from matplotlib.axes import Axes
from matplotlib.colors import Colormap
from matplotlib.typing import MarkerType

from ._core.typing import ColumnName, DataSource, NormSpec
from .axisgrid import FacetGrid
from .utils import _DataSourceWideForm, _ErrorBar, _Estimator, _Legend, _Palette, _Seed, _Vector

__all__ = ["relplot", "scatterplot", "lineplot"]

_Sizes: TypeAlias = list[int] | list[float] | dict[str, int] | dict[str, float] | tuple[float, float]
_DashType: TypeAlias = tuple[None, None] | Sequence[float]  # See matplotlib.lines.Line2D.set_dashes
# "dashes" and "markers" require dict but we use mapping to avoid long unions because dict is invariant in its value type
_Dashes: TypeAlias = bool | Sequence[_DashType] | Mapping[Any, _DashType]
_Markers: TypeAlias = bool | Sequence[MarkerType] | Mapping[Any, MarkerType]

def lineplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    size: ColumnName | _Vector | None = None,
    style: ColumnName | _Vector | None = None,
    units: ColumnName | _Vector | None = None,
    weights: ColumnName | _Vector | None = None,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    hue_norm: NormSpec = None,
    sizes: _Sizes | None = None,
    size_order: Iterable[ColumnName] | None = None,
    size_norm: NormSpec = None,
    dashes: _Dashes | None = True,
    markers: _Markers | None = None,
    style_order: Iterable[ColumnName] | None = None,
    estimator: _Estimator | None = "mean",
    errorbar: _ErrorBar | None = ("ci", 95),
    n_boot: int = 1000,
    seed: _Seed | None = None,
    orient: Literal["x", "y"] = "x",
    sort: bool = True,
    err_style: Literal["band", "bars"] = "band",
    err_kws: dict[str, Any] | None = None,
    legend: _Legend = "auto",
    ci: str | int | None = "deprecated",  # deprecated
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def scatterplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    size: ColumnName | _Vector | None = None,
    style: ColumnName | _Vector | None = None,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    hue_norm: NormSpec = None,
    sizes: _Sizes | None = None,
    size_order: Iterable[ColumnName] | None = None,
    size_norm: NormSpec = None,
    markers: _Markers | None = True,
    style_order: Iterable[ColumnName] | None = None,
    legend: _Legend = "auto",
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def relplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    size: ColumnName | _Vector | None = None,
    style: ColumnName | _Vector | None = None,
    units: ColumnName | _Vector | None = None,
    weights: ColumnName | _Vector | None = None,
    row: ColumnName | _Vector | None = None,
    col: ColumnName | _Vector | None = None,
    col_wrap: int | None = None,
    row_order: Iterable[ColumnName] | None = None,
    col_order: Iterable[ColumnName] | None = None,
    palette: _Palette | Colormap | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    hue_norm: NormSpec = None,
    sizes: _Sizes | None = None,
    size_order: Iterable[ColumnName] | None = None,
    size_norm: NormSpec = None,
    markers: _Markers | None = None,
    dashes: _Dashes | None = None,
    style_order: Iterable[ColumnName] | None = None,
    legend: _Legend = "auto",
    kind: Literal["scatter", "line"] = "scatter",
    height: float = 5,
    aspect: float = 1,
    facet_kws: dict[str, Any] | None = None,
    **kwargs: Any,
) -> FacetGrid: ...



from _typeshed import Incomplete
from typing import Any

from .base import BaseEndpoint as BaseEndpoint

log: Any

class IntrospectEndpoint(BaseEndpoint):
    valid_token_types: Any
    valid_request_methods: Any
    request_validator: Any
    supported_token_types: Any
    def __init__(self, request_validator, supported_token_types: Incomplete | None = None) -> None: ...
    def create_introspect_response(
        self, uri, http_method: str = "POST", body: Incomplete | None = None, headers: Incomplete | None = None
    ): ...
    def validate_introspect_request(self, request) -> None: ...



from typing import Literal, TypeVar

_T = TypeVar("_T")

ints2octs = bytes

def int2oct(x) -> bytes: ...

null: Literal[b""]

def oct2int(x: _T) -> _T: ...
def octs2ints(x: _T) -> _T: ...
def str2octs(x: str) -> bytes: ...
def octs2str(x: bytes) -> str: ...
def isOctetsType(s: object) -> bool: ...
def isStringType(s: object) -> bool: ...

ensureString = bytes



from Xlib._typing import ErrorHandler
from Xlib.display import _BaseDisplay

class Resource:
    display: _BaseDisplay
    id: int
    owner: int
    def __init__(self, display: _BaseDisplay, rid: int, owner: int = 0) -> None: ...
    def __resource__(self) -> int: ...
    def kill_client(self, onerror: ErrorHandler[object] | None = None) -> None: ...



from braintree.search import Search

class CustomerSearch:
    address_extended_address: Search.TextNodeBuilder
    address_first_name: Search.TextNodeBuilder
    address_last_name: Search.TextNodeBuilder
    address_locality: Search.TextNodeBuilder
    address_postal_code: Search.TextNodeBuilder
    address_region: Search.TextNodeBuilder
    address_street_address: Search.TextNodeBuilder
    address_country_name: Search.TextNodeBuilder
    cardholder_name: Search.TextNodeBuilder
    company: Search.TextNodeBuilder
    created_at: Search.RangeNodeBuilder
    credit_card_expiration_date: Search.EqualityNodeBuilder
    credit_card_number: Search.TextNodeBuilder
    email: Search.TextNodeBuilder
    fax: Search.TextNodeBuilder
    first_name: Search.TextNodeBuilder
    id: Search.TextNodeBuilder
    ids: Search.MultipleValueNodeBuilder
    last_name: Search.TextNodeBuilder
    payment_method_token: Search.TextNodeBuilder
    payment_method_token_with_duplicates: Search.IsNodeBuilder
    phone: Search.TextNodeBuilder
    website: Search.TextNodeBuilder
    paypal_account_email: Search.TextNodeBuilder



from _typeshed import Incomplete, StrPath
from collections.abc import Iterable, Iterator
from typing import IO, NoReturn, overload

from . import grammar
from .tokenize import _TokenInfo

class PgenGrammar(grammar.Grammar): ...

class ParserGenerator:
    filename: StrPath
    stream: IO[str]
    generator: Iterator[_TokenInfo]
    first: dict[str, dict[str, int]]
    def __init__(self, filename: StrPath, stream: IO[str] | None = None) -> None: ...
    def make_grammar(self) -> PgenGrammar: ...
    def make_first(self, c: PgenGrammar, name: str) -> dict[int, int]: ...
    def make_label(self, c: PgenGrammar, label: str) -> int: ...
    def addfirstsets(self) -> None: ...
    def calcfirst(self, name: str) -> None: ...
    def parse(self) -> tuple[dict[str, list[DFAState]], str]: ...
    def make_dfa(self, start: NFAState, finish: NFAState) -> list[DFAState]: ...
    def dump_nfa(self, name: str, start: NFAState, finish: NFAState) -> list[DFAState]: ...
    def dump_dfa(self, name: str, dfa: Iterable[DFAState]) -> None: ...
    def simplify_dfa(self, dfa: list[DFAState]) -> None: ...
    def parse_rhs(self) -> tuple[NFAState, NFAState]: ...
    def parse_alt(self) -> tuple[NFAState, NFAState]: ...
    def parse_item(self) -> tuple[NFAState, NFAState]: ...
    def parse_atom(self) -> tuple[NFAState, NFAState]: ...
    def expect(self, type: int, value: str | None = None) -> str: ...
    def gettoken(self) -> None: ...
    @overload
    def raise_error(self, msg: object) -> NoReturn: ...
    @overload
    def raise_error(self, msg: str, *args: object) -> NoReturn: ...

class NFAState:
    arcs: list[tuple[str | None, NFAState]]
    def addarc(self, next: NFAState, label: str | None = None) -> None: ...

class DFAState:
    nfaset: dict[NFAState, Incomplete]
    isfinal: bool
    arcs: dict[str, DFAState]
    def __init__(self, nfaset: dict[NFAState, Incomplete], final: NFAState) -> None: ...
    def addarc(self, next: DFAState, label: str) -> None: ...
    def unifystate(self, old: DFAState, new: DFAState) -> None: ...
    def __eq__(self, other: DFAState) -> bool: ...  # type: ignore[override]

def generate_grammar(filename: StrPath = "Grammar.txt") -> PgenGrammar: ...



from networkx.utils.backends import _dispatch

@_dispatch
def harmonic_function(G, max_iter: int = 30, label_name: str = "label"): ...
@_dispatch
def local_and_global_consistency(G, alpha: float = 0.99, max_iter: int = 30, label_name: str = "label"): ...



from _typeshed import Incomplete

class Configuration:
    @staticmethod
    def configure(environment, merchant_id, public_key, private_key, **kwargs) -> None: ...
    @staticmethod
    def for_partner(environment, partner_id, public_key, private_key, **kwargs): ...
    @staticmethod
    def gateway(): ...
    @staticmethod
    def instantiate(): ...
    @staticmethod
    def api_version(): ...
    @staticmethod
    def graphql_api_version(): ...
    environment: Incomplete
    merchant_id: Incomplete
    public_key: Incomplete
    private_key: Incomplete
    client_id: Incomplete
    client_secret: Incomplete
    access_token: Incomplete
    timeout: Incomplete
    wrap_http_exceptions: Incomplete
    def __init__(
        self,
        environment: Incomplete | None = None,
        merchant_id: Incomplete | None = None,
        public_key: Incomplete | None = None,
        private_key: Incomplete | None = None,
        client_id: Incomplete | None = None,
        client_secret: Incomplete | None = None,
        access_token: Incomplete | None = None,
        *args,
        **kwargs,
    ) -> None: ...
    def base_merchant_path(self): ...
    def base_url(self): ...
    def graphql_base_url(self): ...
    def http(self): ...
    def graphql_client(self): ...
    def http_strategy(self): ...
    def has_client_credentials(self): ...
    def assert_has_client_credentials(self) -> None: ...
    def has_access_token(self): ...



from _typeshed import Incomplete
from typing import ClassVar

import passlib.utils.handlers as uh

class scrypt(uh.ParallelismMixin, uh.HasRounds, uh.HasRawSalt, uh.HasRawChecksum, uh.HasManyIdents, uh.GenericHandler):  # type: ignore[misc]
    backends: ClassVar[tuple[str, ...]]
    name: ClassVar[str]
    checksum_size: ClassVar[int]
    default_ident: ClassVar[str]
    ident_values: ClassVar[tuple[str, ...]]
    default_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    default_rounds: ClassVar[int]
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int]
    rounds_cost: ClassVar[str]
    parallelism: int
    block_size: int
    @classmethod
    def using(cls, block_size: Incomplete | None = None, **kwds): ...  # type: ignore[override]
    @classmethod
    def from_string(cls, hash): ...
    @classmethod
    def parse(cls, hash): ...
    def to_string(self): ...
    def __init__(self, block_size: Incomplete | None = None, **kwds) -> None: ...
    @classmethod
    def get_backend(cls): ...
    @classmethod
    def has_backend(cls, name: str = "any"): ...
    @classmethod
    def set_backend(cls, name: str = "any", dryrun: bool = False) -> None: ...



from _typeshed import Incomplete

class TransactionGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def adjust_authorization(self, transaction_id, amount): ...
    def clone_transaction(self, transaction_id, params): ...
    def cancel_release(self, transaction_id): ...
    def create(self, params): ...
    def credit(self, params): ...
    def find(self, transaction_id): ...
    def hold_in_escrow(self, transaction_id): ...
    def refund(self, transaction_id, amount_or_options: Incomplete | None = None): ...
    def sale(self, params): ...
    def search(self, *query): ...
    def release_from_escrow(self, transaction_id): ...
    def submit_for_settlement(self, transaction_id, amount: Incomplete | None = None, params: Incomplete | None = None): ...
    def update_details(self, transaction_id, params: Incomplete | None = None): ...
    def submit_for_partial_settlement(self, transaction_id, amount, params: Incomplete | None = None): ...
    def package_tracking(self, transaction_id, params: Incomplete | None = None): ...
    def void(self, transaction_id): ...



from collections.abc import Iterable, Iterator
from typing import Any, Literal

from wtforms.meta import _MultiDictLikeWithGetall

def clean_datetime_format_for_strptime(formats: Iterable[str]) -> list[str]: ...

class UnsetValue:
    def __bool__(self) -> Literal[False]: ...

unset_value: UnsetValue

class WebobInputWrapper:
    def __init__(self, multidict: _MultiDictLikeWithGetall) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...
    def __contains__(self, name: str) -> bool: ...
    def getlist(self, name: str) -> list[Any]: ...



import contextlib
from _typeshed import Incomplete, SupportsWrite
from collections.abc import Callable, Iterable, Iterator, Mapping, MutableMapping
from types import TracebackType
from typing import Any, ClassVar, Generic, Literal, NoReturn, TypeVar, overload
from typing_extensions import Self

from ._monitor import TMonitor
from .utils import Comparable

__all__ = [
    "tqdm",
    "trange",
    "TqdmTypeError",
    "TqdmKeyError",
    "TqdmWarning",
    "TqdmExperimentalWarning",
    "TqdmDeprecationWarning",
    "TqdmMonitorWarning",
]

class TqdmTypeError(TypeError): ...
class TqdmKeyError(KeyError): ...

class TqdmWarning(Warning):
    def __init__(self, msg, fp_write: Incomplete | None = None, *a, **k) -> None: ...

class TqdmExperimentalWarning(TqdmWarning, FutureWarning): ...
class TqdmDeprecationWarning(TqdmWarning, DeprecationWarning): ...
class TqdmMonitorWarning(TqdmWarning, RuntimeWarning): ...

_T = TypeVar("_T")

class tqdm(Comparable, Generic[_T]):
    monitor_interval: ClassVar[int]
    monitor: ClassVar[TMonitor | None]

    @staticmethod
    def format_sizeof(num: float, suffix: str = "", divisor: float = 1000) -> str: ...
    @staticmethod
    def format_interval(t: float) -> str: ...
    @staticmethod
    def format_num(n: float) -> str: ...
    @staticmethod
    def status_printer(file: SupportsWrite[str]) -> Callable[[str], None]: ...
    @staticmethod
    def format_meter(
        n: float,
        total: float,
        elapsed: float,
        ncols: int | None = None,
        prefix: str | None = "",
        ascii: bool | str | None = False,
        unit: str | None = "it",
        unit_scale: bool | float | None = False,
        rate: float | None = None,
        bar_format: str | None = None,
        postfix: str | Mapping[str, object] | None = None,
        unit_divisor: float | None = 1000,
        initial: float | None = 0,
        colour: str | None = None,
    ) -> str: ...
    @overload
    def __init__(
        self,
        iterable: Iterable[_T],
        desc: str | None = None,
        total: float | None = None,
        leave: bool | None = True,
        file: SupportsWrite[str] | None = None,
        ncols: int | None = None,
        mininterval: float = 0.1,
        maxinterval: float = 10.0,
        miniters: float | None = None,
        ascii: bool | str | None = None,
        disable: bool | None = False,
        unit: str = "it",
        unit_scale: bool | float = False,
        dynamic_ncols: bool = False,
        smoothing: float = 0.3,
        bar_format: str | None = None,
        initial: float = 0,
        position: int | None = None,
        postfix: Mapping[str, object] | str | None = None,
        unit_divisor: float = 1000,
        write_bytes: bool = False,
        lock_args: tuple[bool | None, float | None] | tuple[bool | None] | None = None,
        nrows: int | None = None,
        colour: str | None = None,
        delay: float | None = 0,
        gui: bool = False,
        **kwargs,
    ) -> None: ...
    @overload
    def __init__(
        self: tqdm[NoReturn],
        iterable: None = None,
        desc: str | None = None,
        total: float | None = None,
        leave: bool | None = True,
        file: SupportsWrite[str] | None = None,
        ncols: int | None = None,
        mininterval: float = 0.1,
        maxinterval: float = 10.0,
        miniters: float | None = None,
        ascii: bool | str | None = None,
        disable: bool | None = False,
        unit: str = "it",
        unit_scale: bool | float = False,
        dynamic_ncols: bool = False,
        smoothing: float = 0.3,
        bar_format: str | None = None,
        initial: float = 0,
        position: int | None = None,
        postfix: Mapping[str, object] | str | None = None,
        unit_divisor: float = 1000,
        write_bytes: bool | None = False,
        lock_args: tuple[bool | None, float | None] | tuple[bool | None] | None = None,
        nrows: int | None = None,
        colour: str | None = None,
        delay: float | None = 0,
        gui: bool = False,
        **kwargs,
    ) -> None: ...
    def __new__(cls, *_, **__) -> Self: ...
    @classmethod
    def write(cls, s: str, file: SupportsWrite[str] | None = None, end: str = "\n", nolock: bool = False) -> None: ...
    @classmethod
    def external_write_mode(
        cls, file: SupportsWrite[str] | None = None, nolock: bool = False
    ) -> contextlib._GeneratorContextManager[None]: ...
    @classmethod
    def set_lock(cls, lock) -> None: ...
    @classmethod
    def get_lock(cls): ...
    @classmethod
    def pandas(
        cls,
        *,
        desc: str | None = ...,
        total: float | None = ...,
        leave: bool | None = ...,
        file: SupportsWrite[str] | None = ...,
        ncols: int | None = ...,
        mininterval: float = ...,
        maxinterval: float = ...,
        miniters: float | None = ...,
        ascii: bool | str | None = ...,
        disable: bool | None = ...,
        unit: str = ...,
        unit_scale: bool | float = ...,
        dynamic_ncols: bool = ...,
        smoothing: float = ...,
        bar_format: str | None = ...,
        initial: float = ...,
        position: int | None = ...,
        postfix: Mapping[str, object] | str | None = ...,
        unit_divisor: float = ...,
        write_bytes: bool | None = ...,
        lock_args: tuple[bool | None, float | None] | tuple[bool | None] | None = ...,
        nrows: int | None = ...,
        colour: str | None = ...,
        delay: float | None = ...,
    ) -> None: ...

    iterable: Incomplete
    disable: Incomplete
    pos: Incomplete
    n: Incomplete
    total: Incomplete
    leave: Incomplete
    desc: Incomplete
    fp: Incomplete
    ncols: Incomplete
    nrows: Incomplete
    mininterval: Incomplete
    maxinterval: Incomplete
    miniters: Incomplete
    dynamic_miniters: Incomplete
    ascii: Incomplete
    unit: Incomplete
    unit_scale: Incomplete
    unit_divisor: Incomplete
    initial: Incomplete
    lock_args: Incomplete
    delay: Incomplete
    gui: Incomplete
    dynamic_ncols: Incomplete
    smoothing: Incomplete
    bar_format: Incomplete
    postfix: Incomplete
    colour: Incomplete
    last_print_n: Incomplete
    sp: Incomplete
    last_print_t: Incomplete
    start_t: Incomplete

    def __bool__(self) -> bool: ...
    def __len__(self) -> int: ...
    def __reversed__(self) -> Iterator[_T]: ...
    def __contains__(self, item: object) -> bool: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __del__(self) -> None: ...
    def __hash__(self) -> int: ...
    def __iter__(self) -> Iterator[_T]: ...
    def update(self, n: float | None = 1) -> bool | None: ...
    def close(self) -> None: ...
    def clear(self, nolock: bool = False) -> None: ...
    def refresh(
        self, nolock: bool = False, lock_args: tuple[bool | None, float | None] | tuple[bool | None] | None = None
    ) -> None: ...
    def unpause(self) -> None: ...
    def reset(self, total: float | None = None) -> None: ...
    def set_description(self, desc: str | None = None, refresh: bool | None = True) -> None: ...
    def set_description_str(self, desc: str | None = None, refresh: bool | None = True) -> None: ...
    def set_postfix(self, ordered_dict: Mapping[str, object] | None = None, refresh: bool | None = True, **kwargs) -> None: ...
    def set_postfix_str(self, s: str = "", refresh: bool = True) -> None: ...
    def moveto(self, n) -> None: ...
    @property
    def format_dict(self) -> MutableMapping[str, Any]: ...
    def display(self, msg: str | None = None, pos: int | None = None) -> None: ...
    @classmethod
    def wrapattr(
        cls, stream, method: Literal["read", "write"], total: float | None = None, bytes: bool | None = True, **tqdm_kwargs
    ) -> contextlib._GeneratorContextManager[Incomplete]: ...

@overload
def trange(
    start: int,
    stop: int,
    step: int | None = ...,
    *,
    desc: str | None = ...,
    total: float | None = ...,
    leave: bool | None = ...,
    file: SupportsWrite[str] | None = ...,
    ncols: int | None = ...,
    mininterval: float = ...,
    maxinterval: float = ...,
    miniters: float | None = ...,
    ascii: bool | str | None = ...,
    disable: bool | None = ...,
    unit: str = ...,
    unit_scale: bool | float = ...,
    dynamic_ncols: bool = ...,
    smoothing: float = ...,
    bar_format: str | None = ...,
    initial: float = ...,
    position: int | None = ...,
    postfix: Mapping[str, object] | str | None = ...,
    unit_divisor: float = ...,
    write_bytes: bool | None = ...,
    lock_args: tuple[bool | None, float | None] | tuple[bool | None] | None = ...,
    nrows: int | None = ...,
    colour: str | None = ...,
    delay: float | None = ...,
) -> tqdm[int]: ...
@overload
def trange(
    stop: int,
    *,
    desc: str | None = ...,
    total: float | None = ...,
    leave: bool | None = ...,
    file: SupportsWrite[str] | None = ...,
    ncols: int | None = ...,
    mininterval: float = ...,
    maxinterval: float = ...,
    miniters: float | None = ...,
    ascii: bool | str | None = ...,
    disable: bool | None = ...,
    unit: str = ...,
    unit_scale: bool | float = ...,
    dynamic_ncols: bool = ...,
    smoothing: float = ...,
    bar_format: str | None = ...,
    initial: float = ...,
    position: int | None = ...,
    postfix: Mapping[str, object] | str | None = ...,
    unit_divisor: float = ...,
    write_bytes: bool | None = ...,
    lock_args: tuple[bool | None, float | None] | tuple[bool | None] | None = ...,
    nrows: int | None = ...,
    colour: str | None = ...,
    delay: float | None = ...,
) -> tqdm[int]: ...



def require_string(value, value_name) -> None: ...
def require_callback(callback, callback_name: str = "callback") -> None: ...
def rpc_completion_callback(callback): ...
def zero_or_greater(name, value) -> None: ...



from _typeshed import Incomplete

class Run:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        id: Incomplete | None = None,
        task_id: Incomplete | None = None,
        status: Incomplete | None = None,
        scheduled_for: Incomplete | None = None,
        log: Incomplete | None = None,
        flux: str | None = None,
        started_at: Incomplete | None = None,
        finished_at: Incomplete | None = None,
        requested_at: Incomplete | None = None,
        links: Incomplete | None = None,
    ) -> None: ...
    @property
    def id(self): ...
    @id.setter
    def id(self, id) -> None: ...
    @property
    def task_id(self): ...
    @task_id.setter
    def task_id(self, task_id) -> None: ...
    @property
    def status(self): ...
    @status.setter
    def status(self, status) -> None: ...
    @property
    def scheduled_for(self): ...
    @scheduled_for.setter
    def scheduled_for(self, scheduled_for) -> None: ...
    @property
    def log(self): ...
    @log.setter
    def log(self, log) -> None: ...
    flux: str | None
    @property
    def started_at(self): ...
    @started_at.setter
    def started_at(self, started_at) -> None: ...
    @property
    def finished_at(self): ...
    @finished_at.setter
    def finished_at(self, finished_at) -> None: ...
    @property
    def requested_at(self): ...
    @requested_at.setter
    def requested_at(self, requested_at) -> None: ...
    @property
    def links(self): ...
    @links.setter
    def links(self, links) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete

from hvac.api.auth_methods.jwt import JWT

class OIDC(JWT):
    DEFAULT_PATH: str
    def create_role(
        self,
        name,
        user_claim,
        allowed_redirect_uris,
        role_type: str = "oidc",
        bound_audiences: Incomplete | None = None,
        clock_skew_leeway: Incomplete | None = None,
        expiration_leeway: Incomplete | None = None,
        not_before_leeway: Incomplete | None = None,
        bound_subject: Incomplete | None = None,
        bound_claims: Incomplete | None = None,
        groups_claim: Incomplete | None = None,
        claim_mappings: Incomplete | None = None,
        oidc_scopes: Incomplete | None = None,
        bound_claims_type: str = "string",
        verbose_oidc_logging: bool = False,
        token_ttl: Incomplete | None = None,
        token_max_ttl: Incomplete | None = None,
        token_policies: Incomplete | None = None,
        token_bound_cidrs: Incomplete | None = None,
        token_explicit_max_ttl: Incomplete | None = None,
        token_no_default_policy: Incomplete | None = None,
        token_num_uses: Incomplete | None = None,
        token_period: Incomplete | None = None,
        token_type: Incomplete | None = None,
        path: Incomplete | None = None,
        user_claim_json_pointer: Incomplete | None = None,
    ) -> None: ...



from _socket import _Address as _StrictAddress
from _typeshed import ReadableBuffer, StrOrBytesPath
from collections.abc import Callable
from typing import Any, ClassVar, TypedDict, overload
from typing_extensions import TypeAlias

from gevent.baseserver import BaseServer, _Spawner
from gevent.socket import socket as _GeventSocket
from gevent.ssl import SSLContext, wrap_socket as ssl_wrap_socket

# For simplicity we treat _Address as Any, we could be more strict and use the definition
# from the stdlib _socket.pyi. But that would exclude some potentially valid handlers.
_Address: TypeAlias = Any

class _SSLArguments(TypedDict, total=False):
    keyfile: StrOrBytesPath
    certfile: StrOrBytesPath
    server_side: bool
    cert_reqs: int
    ssl_version: int
    ca_certs: str
    suppress_ragged_eofs: bool
    do_handshake_on_connect: bool
    ciphers: str

class StreamServer(BaseServer[[_GeventSocket, _Address]]):
    backlog: int
    reuse_addr: ClassVar[int | None]
    wrap_socket = ssl_wrap_socket
    ssl_args: _SSLArguments | None
    @overload
    def __init__(
        self,
        listener: _GeventSocket | tuple[str, int] | str,
        handle: Callable[[_GeventSocket, _Address], object] | None = None,
        backlog: int | None = None,
        spawn: _Spawner = "default",
        *,
        ssl_context: SSLContext,
        server_side: bool = True,
        do_handshake_on_connect: bool = True,
        suppress_ragged_eofs: bool = True,
    ) -> None: ...
    @overload
    def __init__(
        self,
        listener: _GeventSocket | tuple[str, int] | str,
        handle: Callable[[_GeventSocket, _Address], object] | None = None,
        backlog: int | None = None,
        spawn: _Spawner = "default",
        *,
        keyfile: StrOrBytesPath = ...,
        certfile: StrOrBytesPath = ...,
        server_side: bool = True,
        cert_reqs: int = ...,
        ssl_version: int = ...,
        ca_certs: str = ...,
        do_handshake_on_connect: bool = True,
        suppress_ragged_eofs: bool = True,
        ciphers: str = ...,
    ) -> None: ...
    @property
    def ssl_enabled(self) -> bool: ...
    @classmethod
    def get_listener(cls, address: _StrictAddress, backlog: int | None = None, family: int | None = None) -> _GeventSocket: ...
    def do_read(self) -> tuple[_GeventSocket, _Address]: ...
    def do_close(self, sock: _GeventSocket, address: _Address) -> None: ...
    def wrap_socket_and_handle(self, client_socket: _GeventSocket, address: _StrictAddress) -> Any: ...

class DatagramServer(BaseServer[[_GeventSocket, _Address]]):
    reuse_addr: ClassVar[int | None]
    def __init__(
        self,
        listener: _GeventSocket | tuple[str, int] | str,
        handle: Callable[[_GeventSocket, _Address], object] | None = None,
        spawn: _Spawner = "default",
    ) -> None: ...
    @classmethod
    def get_listener(cls, address: _StrictAddress, family: int | None = None) -> _GeventSocket: ...
    def do_read(self) -> tuple[_GeventSocket, _Address]: ...
    @overload
    def sendto(self, data: ReadableBuffer, address: _StrictAddress, /) -> int: ...
    @overload
    def sendto(self, data: ReadableBuffer, flags: int, address: _StrictAddress, /) -> int: ...



from collections.abc import Callable
from types import TracebackType
from typing import Any, Literal, TypeVar, overload
from typing_extensions import ParamSpec, Self

from gevent._types import _TimerWatcher

_T = TypeVar("_T")
_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2")
_TimeoutT = TypeVar("_TimeoutT", bound=Timeout)
_P = ParamSpec("_P")

class Timeout(BaseException):
    seconds: float | None
    exception: type[BaseException] | BaseException | None
    timer: _TimerWatcher
    def __init__(
        self,
        seconds: float | None = None,
        exception: type[BaseException] | BaseException | None = None,
        ref: bool = True,
        priority: int = -1,
    ) -> None: ...
    def start(self) -> None: ...
    @overload
    @classmethod
    def start_new(
        cls, timeout: None | float = None, exception: type[BaseException] | BaseException | None = None, ref: bool = True
    ) -> Self: ...
    @overload
    @classmethod
    def start_new(cls, timeout: _TimeoutT) -> _TimeoutT: ...
    @property
    def pending(self) -> bool: ...
    def cancel(self) -> None: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, typ: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None
    ) -> Literal[True] | None: ...

# when timeout_value is provided we unfortunately get no type checking on *args, **kwargs, because
# ParamSpec does not allow mixing in additional keyword arguments
@overload
def with_timeout(seconds: float | None, function: Callable[..., _T1], *args: Any, timeout_value: _T2, **kwds: Any) -> _T1 | _T2: ...  # type: ignore[misc]
@overload
def with_timeout(seconds: float | None, function: Callable[_P, _T], *args: _P.args, **kwds: _P.kwargs) -> _T: ...



from collections.abc import Generator, Hashable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatch

@_dispatch
def is_k_edge_connected(G: Graph[Hashable], k: int): ...
@_dispatch
def is_locally_k_edge_connected(G, s, t, k): ...
@_dispatch
def k_edge_augmentation(
    G: Graph[_Node],
    k: int,
    avail: tuple[_Node, _Node] | tuple[_Node, _Node, dict[str, int]] | None = None,
    weight: str | None = None,
    partial: bool = False,
) -> Generator[tuple[_Node, _Node], None, None]: ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.utils.backends import _dispatch

@_dispatch
def dijkstra_path(G, source, target, weight: str = "weight"): ...
@_dispatch
def dijkstra_path_length(G, source, target, weight: str = "weight"): ...
@_dispatch
def single_source_dijkstra_path(G, source, cutoff: Incomplete | None = None, weight: str = "weight"): ...
@_dispatch
def single_source_dijkstra_path_length(G, source, cutoff: Incomplete | None = None, weight: str = "weight"): ...
@_dispatch
def single_source_dijkstra(
    G, source, target: Incomplete | None = None, cutoff: Incomplete | None = None, weight: str = "weight"
): ...
@_dispatch
def multi_source_dijkstra_path(G, sources, cutoff: Incomplete | None = None, weight: str = "weight"): ...
@_dispatch
def multi_source_dijkstra_path_length(G, sources, cutoff: Incomplete | None = None, weight: str = "weight"): ...
@_dispatch
def multi_source_dijkstra(
    G, sources, target: Incomplete | None = None, cutoff: Incomplete | None = None, weight: str = "weight"
): ...
@_dispatch
def dijkstra_predecessor_and_distance(G, source, cutoff: Incomplete | None = None, weight: str = "weight"): ...
@_dispatch
def all_pairs_dijkstra(G, cutoff: Incomplete | None = None, weight: str = "weight") -> Generator[Incomplete, None, None]: ...
@_dispatch
def all_pairs_dijkstra_path_length(
    G, cutoff: Incomplete | None = None, weight: str = "weight"
) -> Generator[Incomplete, None, None]: ...
@_dispatch
def all_pairs_dijkstra_path(G, cutoff: Incomplete | None = None, weight: str = "weight") -> Generator[Incomplete, None, None]: ...
@_dispatch
def bellman_ford_predecessor_and_distance(
    G, source, target: Incomplete | None = None, weight: str = "weight", heuristic: bool = False
): ...
@_dispatch
def bellman_ford_path(G, source, target, weight: str = "weight"): ...
@_dispatch
def bellman_ford_path_length(G, source, target, weight: str = "weight"): ...
@_dispatch
def single_source_bellman_ford_path(G, source, weight: str = "weight"): ...
@_dispatch
def single_source_bellman_ford_path_length(G, source, weight: str = "weight"): ...
@_dispatch
def single_source_bellman_ford(G, source, target: Incomplete | None = None, weight: str = "weight"): ...
@_dispatch
def all_pairs_bellman_ford_path_length(G, weight: str = "weight") -> Generator[Incomplete, None, None]: ...
@_dispatch
def all_pairs_bellman_ford_path(G, weight: str = "weight") -> Generator[Incomplete, None, None]: ...
@_dispatch
def goldberg_radzik(G, source, weight: str = "weight"): ...
@_dispatch
def negative_edge_cycle(G, weight: str = "weight", heuristic: bool = True): ...
@_dispatch
def find_negative_cycle(G, source, weight: str = "weight"): ...
@_dispatch
def bidirectional_dijkstra(G, source, target, weight: str = "weight"): ...
@_dispatch
def johnson(G, weight: str = "weight"): ...



from typing import Final

PRIMARY: Final = 1
SECONDARY: Final = 2
ARC: Final = 3
ATOM: Final = 4
BITMAP: Final = 5
CARDINAL: Final = 6
COLORMAP: Final = 7
CURSOR: Final = 8
CUT_BUFFER0: Final = 9
CUT_BUFFER1: Final = 10
CUT_BUFFER2: Final = 11
CUT_BUFFER3: Final = 12
CUT_BUFFER4: Final = 13
CUT_BUFFER5: Final = 14
CUT_BUFFER6: Final = 15
CUT_BUFFER7: Final = 16
DRAWABLE: Final = 17
FONT: Final = 18
INTEGER: Final = 19
PIXMAP: Final = 20
POINT: Final = 21
RECTANGLE: Final = 22
RESOURCE_MANAGER: Final = 23
RGB_COLOR_MAP: Final = 24
RGB_BEST_MAP: Final = 25
RGB_BLUE_MAP: Final = 26
RGB_DEFAULT_MAP: Final = 27
RGB_GRAY_MAP: Final = 28
RGB_GREEN_MAP: Final = 29
RGB_RED_MAP: Final = 30
STRING: Final = 31
VISUALID: Final = 32
WINDOW: Final = 33
WM_COMMAND: Final = 34
WM_HINTS: Final = 35
WM_CLIENT_MACHINE: Final = 36
WM_ICON_NAME: Final = 37
WM_ICON_SIZE: Final = 38
WM_NAME: Final = 39
WM_NORMAL_HINTS: Final = 40
WM_SIZE_HINTS: Final = 41
WM_ZOOM_HINTS: Final = 42
MIN_SPACE: Final = 43
NORM_SPACE: Final = 44
MAX_SPACE: Final = 45
END_SPACE: Final = 46
SUPERSCRIPT_X: Final = 47
SUPERSCRIPT_Y: Final = 48
SUBSCRIPT_X: Final = 49
SUBSCRIPT_Y: Final = 50
UNDERLINE_POSITION: Final = 51
UNDERLINE_THICKNESS: Final = 52
STRIKEOUT_ASCENT: Final = 53
STRIKEOUT_DESCENT: Final = 54
ITALIC_ANGLE: Final = 55
X_HEIGHT: Final = 56
QUAD_WIDTH: Final = 57
WEIGHT: Final = 58
POINT_SIZE: Final = 59
RESOLUTION: Final = 60
COPYRIGHT: Final = 61
NOTICE: Final = 62
FONT_NAME: Final = 63
FAMILY_NAME: Final = 64
FULL_NAME: Final = 65
CAP_HEIGHT: Final = 66
WM_CLASS: Final = 67
WM_TRANSIENT_FOR: Final = 68
LAST_PREDEFINED: Final = 68



from typing import ClassVar

from .core import UnitedStates

class Virginia(UnitedStates):
    include_christmas_eve: ClassVar[bool]
    include_thanksgiving_friday: ClassVar[bool]
    include_boxing_day: ClassVar[bool]
    presidents_day_label: ClassVar[str]
    include_thanksgiving_wednesday: ClassVar[bool]
    def get_variable_days(self, year): ...



from typing import ClassVar

from .core import UnitedStates

class NorthCarolina(UnitedStates):
    include_good_friday: ClassVar[bool]
    include_christmas_eve: ClassVar[bool]
    include_thanksgiving_friday: ClassVar[bool]
    include_boxing_day: ClassVar[bool]
    include_federal_presidents_day: ClassVar[bool]
    include_columbus_day: ClassVar[bool]
    def get_christmas_shifts(self, year): ...
    def get_variable_days(self, year): ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import ChineseNewYearCalendar, WesternMixin

class HongKong(WesternMixin, ChineseNewYearCalendar):
    include_labour_day: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    include_easter_saturday: ClassVar[bool]
    include_easter_monday: ClassVar[bool]
    include_boxing_day: ClassVar[bool]
    WEEKEND_DAYS: Incomplete
    FIXED_HOLIDAYS: Incomplete
    chinese_new_year_label: ClassVar[str]
    include_chinese_second_day: ClassVar[bool]
    chinese_second_day_label: ClassVar[str]
    include_chinese_third_day: ClassVar[bool]
    chinese_third_day_label: ClassVar[str]
    shift_sunday_holidays: ClassVar[bool]
    shift_start_cny_sunday: ClassVar[bool]
    def get_variable_days(self, year): ...

class HongKongBank(HongKong):
    WEEKEND_DAYS: Incomplete



from _typeshed import ConvertibleToInt, Incomplete, Unused
from typing import ClassVar, Literal

from openpyxl.descriptors.base import Bool, Integer, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

class ChartsheetView(Serialisable):
    tagname: ClassVar[str]
    tabSelected: Bool[Literal[True]]
    zoomScale: Integer[Literal[True]]
    workbookViewId: Integer[Literal[False]]
    zoomToFit: Bool[Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        tabSelected: _ConvertibleToBool | None = None,
        zoomScale: ConvertibleToInt | None = None,
        workbookViewId: ConvertibleToInt = 0,
        zoomToFit: _ConvertibleToBool | None = True,
        extLst: Unused = None,
    ) -> None: ...

class ChartsheetViewList(Serialisable):
    tagname: ClassVar[str]
    sheetView: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, sheetView: Incomplete | None = None, extLst: Unused = None) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator

from .. import Command

def strip_module(filename): ...
def sorted_walk(dir) -> Generator[Incomplete, None, None]: ...
def write_stub(resource, pyfile) -> None: ...

class bdist_egg(Command):
    description: str
    user_options: Incomplete
    boolean_options: Incomplete
    bdist_dir: Incomplete
    plat_name: Incomplete
    keep_temp: int
    dist_dir: Incomplete
    skip_build: int
    egg_output: Incomplete
    exclude_source_files: Incomplete
    def initialize_options(self) -> None: ...
    egg_info: Incomplete
    def finalize_options(self) -> None: ...
    def do_install_data(self) -> None: ...
    def get_outputs(self): ...
    def call_command(self, cmdname, **kw): ...
    stubs: Incomplete
    def run(self) -> None: ...
    def zap_pyfiles(self) -> None: ...
    def zip_safe(self): ...
    def gen_header(self): ...
    def copy_metadata_to(self, target_dir) -> None: ...
    def get_ext_outputs(self): ...

NATIVE_EXTENSIONS: Incomplete

def walk_egg(egg_dir) -> Generator[Incomplete, None, None]: ...
def analyze_egg(egg_dir, stubs): ...
def write_safety_flag(egg_dir, safe) -> None: ...

safety_flags: Incomplete

def scan_module(egg_dir, base, name, stubs): ...
def iter_symbols(code) -> Generator[Incomplete, None, None]: ...
def can_scan(): ...

INSTALL_DIRECTORY_ATTRS: Incomplete

def make_zipfile(zip_filename, base_dir, verbose: int = 0, dry_run: int = 0, compress: bool = True, mode: str = "w"): ...



from _typeshed import Incomplete
from logging import Logger
from typing import ClassVar
from typing_extensions import TypeAlias

from .spec import Connection

# TODO: This could be turned into a protocol.
_Credentials: TypeAlias = Incomplete  # noqa: Y047

LOGGER: Logger

class PlainCredentials:
    TYPE: ClassVar[str]
    username: str
    password: str
    erase_on_connect: bool
    def __init__(self, username: str, password: str, erase_on_connect: bool = False) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def response_for(self, start: Connection.Start) -> tuple[str | None, bytes | None]: ...
    def erase_credentials(self) -> None: ...

class ExternalCredentials:
    TYPE: ClassVar[str]
    erase_on_connect: bool
    def __init__(self) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def response_for(self, start: Connection.Start) -> tuple[str | None, bytes | None]: ...
    def erase_credentials(self) -> None: ...

VALID_TYPES: Incomplete



from typing import ClassVar

class AutumHoliday:
    include_autumn_holiday: ClassVar[bool]
    autumn_holiday_label: ClassVar[str]

class AutumnHolidayLastMondaySeptember(AutumHoliday):
    def get_autumn_holiday(self, year): ...

class AutumnHolidayFirstMondayOctober(AutumHoliday):
    def get_autumn_holiday(self, year): ...

class AutumnHolidaySecondMondayOctober(AutumHoliday):
    def get_autumn_holiday(self, year): ...

class AutumnHolidayThirdMondayOctober(AutumHoliday):
    def get_autumn_holiday(self, year): ...



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class NotificationEndpointsService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def create_notification_endpoint(self, post_notification_endpoint, **kwargs): ...
    def create_notification_endpoint_with_http_info(self, post_notification_endpoint, **kwargs): ...
    async def create_notification_endpoint_async(self, post_notification_endpoint, **kwargs): ...
    def delete_notification_endpoints_id(self, endpoint_id, **kwargs): ...
    def delete_notification_endpoints_id_with_http_info(self, endpoint_id, **kwargs): ...
    async def delete_notification_endpoints_id_async(self, endpoint_id, **kwargs): ...
    def delete_notification_endpoints_id_labels_id(self, endpoint_id, label_id, **kwargs): ...
    def delete_notification_endpoints_id_labels_id_with_http_info(self, endpoint_id, label_id, **kwargs): ...
    async def delete_notification_endpoints_id_labels_id_async(self, endpoint_id, label_id, **kwargs): ...
    def get_notification_endpoints(self, org_id, **kwargs): ...
    def get_notification_endpoints_with_http_info(self, org_id, **kwargs): ...
    async def get_notification_endpoints_async(self, org_id, **kwargs): ...
    def get_notification_endpoints_id(self, endpoint_id, **kwargs): ...
    def get_notification_endpoints_id_with_http_info(self, endpoint_id, **kwargs): ...
    async def get_notification_endpoints_id_async(self, endpoint_id, **kwargs): ...
    def get_notification_endpoints_id_labels(self, endpoint_id, **kwargs): ...
    def get_notification_endpoints_id_labels_with_http_info(self, endpoint_id, **kwargs): ...
    async def get_notification_endpoints_id_labels_async(self, endpoint_id, **kwargs): ...
    def patch_notification_endpoints_id(self, endpoint_id, notification_endpoint_update, **kwargs): ...
    def patch_notification_endpoints_id_with_http_info(self, endpoint_id, notification_endpoint_update, **kwargs): ...
    async def patch_notification_endpoints_id_async(self, endpoint_id, notification_endpoint_update, **kwargs): ...
    def post_notification_endpoint_id_labels(self, endpoint_id, label_mapping, **kwargs): ...
    def post_notification_endpoint_id_labels_with_http_info(self, endpoint_id, label_mapping, **kwargs): ...
    async def post_notification_endpoint_id_labels_async(self, endpoint_id, label_mapping, **kwargs): ...
    def put_notification_endpoints_id(self, endpoint_id, notification_endpoint, **kwargs): ...
    def put_notification_endpoints_id_with_http_info(self, endpoint_id, notification_endpoint, **kwargs): ...
    async def put_notification_endpoints_id_async(self, endpoint_id, notification_endpoint, **kwargs): ...



from .enums import PageMode

class ViewerPreferences:
    hide_toolbar: bool
    hide_menubar: bool
    hide_window_u_i: bool
    fit_window: bool
    center_window: bool
    display_doc_title: bool
    def __init__(
        self,
        hide_toolbar: bool = False,
        hide_menubar: bool = False,
        hide_window_u_i: bool = False,
        fit_window: bool = False,
        center_window: bool = False,
        display_doc_title: bool = False,
        non_full_screen_page_mode: PageMode | str = ...,
    ) -> None: ...
    @property
    def non_full_screen_page_mode(self): ...
    @non_full_screen_page_mode.setter
    def non_full_screen_page_mode(self, page_mode) -> None: ...
    def serialize(self) -> str: ...



from collections.abc import Callable
from typing import AnyStr, Protocol

class _HasHTML(Protocol):
    def __html__(self) -> str: ...

def html_escape(s: str | bytes | _HasHTML) -> str: ...
def header_docstring(header: str, rfc_section: str) -> str: ...
def warn_deprecation(text: str, version: str, stacklevel: int) -> None: ...

status_reasons: dict[int, str]
status_generic_reasons: dict[int, str]

def strings_differ(string1: AnyStr, string2: AnyStr, compare_digest: Callable[[AnyStr, AnyStr], bool] = ...) -> bool: ...



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class ScraperTargetsService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def delete_scrapers_id(self, scraper_target_id, **kwargs): ...
    def delete_scrapers_id_with_http_info(self, scraper_target_id, **kwargs): ...
    async def delete_scrapers_id_async(self, scraper_target_id, **kwargs): ...
    def delete_scrapers_id_labels_id(self, scraper_target_id, label_id, **kwargs): ...
    def delete_scrapers_id_labels_id_with_http_info(self, scraper_target_id, label_id, **kwargs): ...
    async def delete_scrapers_id_labels_id_async(self, scraper_target_id, label_id, **kwargs): ...
    def delete_scrapers_id_members_id(self, user_id, scraper_target_id, **kwargs): ...
    def delete_scrapers_id_members_id_with_http_info(self, user_id, scraper_target_id, **kwargs): ...
    async def delete_scrapers_id_members_id_async(self, user_id, scraper_target_id, **kwargs): ...
    def delete_scrapers_id_owners_id(self, user_id, scraper_target_id, **kwargs): ...
    def delete_scrapers_id_owners_id_with_http_info(self, user_id, scraper_target_id, **kwargs): ...
    async def delete_scrapers_id_owners_id_async(self, user_id, scraper_target_id, **kwargs): ...
    def get_scrapers(self, **kwargs): ...
    def get_scrapers_with_http_info(self, **kwargs): ...
    async def get_scrapers_async(self, **kwargs): ...
    def get_scrapers_id(self, scraper_target_id, **kwargs): ...
    def get_scrapers_id_with_http_info(self, scraper_target_id, **kwargs): ...
    async def get_scrapers_id_async(self, scraper_target_id, **kwargs): ...
    def get_scrapers_id_labels(self, scraper_target_id, **kwargs): ...
    def get_scrapers_id_labels_with_http_info(self, scraper_target_id, **kwargs): ...
    async def get_scrapers_id_labels_async(self, scraper_target_id, **kwargs): ...
    def get_scrapers_id_members(self, scraper_target_id, **kwargs): ...
    def get_scrapers_id_members_with_http_info(self, scraper_target_id, **kwargs): ...
    async def get_scrapers_id_members_async(self, scraper_target_id, **kwargs): ...
    def get_scrapers_id_owners(self, scraper_target_id, **kwargs): ...
    def get_scrapers_id_owners_with_http_info(self, scraper_target_id, **kwargs): ...
    async def get_scrapers_id_owners_async(self, scraper_target_id, **kwargs): ...
    def patch_scrapers_id(self, scraper_target_id, scraper_target_request, **kwargs): ...
    def patch_scrapers_id_with_http_info(self, scraper_target_id, scraper_target_request, **kwargs): ...
    async def patch_scrapers_id_async(self, scraper_target_id, scraper_target_request, **kwargs): ...
    def post_scrapers(self, scraper_target_request, **kwargs): ...
    def post_scrapers_with_http_info(self, scraper_target_request, **kwargs): ...
    async def post_scrapers_async(self, scraper_target_request, **kwargs): ...
    def post_scrapers_id_labels(self, scraper_target_id, label_mapping, **kwargs): ...
    def post_scrapers_id_labels_with_http_info(self, scraper_target_id, label_mapping, **kwargs): ...
    async def post_scrapers_id_labels_async(self, scraper_target_id, label_mapping, **kwargs): ...
    def post_scrapers_id_members(self, scraper_target_id, add_resource_member_request_body, **kwargs): ...
    def post_scrapers_id_members_with_http_info(self, scraper_target_id, add_resource_member_request_body, **kwargs): ...
    async def post_scrapers_id_members_async(self, scraper_target_id, add_resource_member_request_body, **kwargs): ...
    def post_scrapers_id_owners(self, scraper_target_id, add_resource_member_request_body, **kwargs): ...
    def post_scrapers_id_owners_with_http_info(self, scraper_target_id, add_resource_member_request_body, **kwargs): ...
    async def post_scrapers_id_owners_async(self, scraper_target_id, add_resource_member_request_body, **kwargs): ...



from re import Pattern
from typing import Any, ClassVar
from typing_extensions import TypeGuard
from xml.etree.ElementTree import Element

from markdown import util
from markdown.core import Markdown

def build_treeprocessors(md: Markdown, **kwargs) -> util.Registry[Treeprocessor]: ...
def isString(s: object) -> TypeGuard[str]: ...

class Treeprocessor(util.Processor):
    def run(self, root: Element) -> Element | None: ...

class InlineProcessor(Treeprocessor):
    inlinePatterns: util.Registry[InlineProcessor]
    ancestors: list[str]
    def __init__(self, md: Markdown) -> None: ...
    stashed_nodes: dict[str, Element | str]
    parent_map: Any
    def run(self, tree: Element, ancestors: list[str] | None = None) -> Element: ...

class PrettifyTreeprocessor(Treeprocessor): ...

class UnescapeTreeprocessor(Treeprocessor):
    RE: ClassVar[Pattern[str]]
    def unescape(self, text: str) -> str: ...



from _typeshed import Incomplete
from typing import Any

from .segment import Segment

MUTATION_UNSUPPORTED_MESSAGE: str

class FacadeSegment(Segment):
    initializing: Any
    def __init__(self, name, entityid, traceid, sampled) -> None: ...
    def close(self, end_time: Incomplete | None = None) -> None: ...
    def put_http_meta(self, key, value) -> None: ...
    def put_annotation(self, key, value) -> None: ...
    def put_metadata(self, key, value, namespace: str = "default") -> None: ...
    def set_aws(self, aws_meta) -> None: ...
    def set_user(self, user) -> None: ...
    def add_throttle_flag(self) -> None: ...
    def add_fault_flag(self) -> None: ...
    def add_error_flag(self) -> None: ...
    def add_exception(self, exception, stack, remote: bool = False) -> None: ...
    def apply_status_code(self, status_code) -> None: ...
    def serialize(self) -> None: ...
    def ready_to_send(self): ...
    def increment(self) -> None: ...
    def decrement_ref_counter(self) -> None: ...



from typing import Any
from typing_extensions import Self

from .base import Key

LEGACY_INVALID_PKCS8_RSA_HEADER: bytes
ASN1_SEQUENCE_ID: bytes
RSA_ENCRYPTION_ASN1_OID: str

# Enable when we can use stubs from installed dependencies:
# from rsa import PublicKey
def pem_to_spki(pem, fmt: str = "PKCS8"): ...

class RSAKey(Key):
    SHA256: str
    SHA384: str
    SHA512: str
    hash_alg: str
    def __init__(self, key, algorithm) -> None: ...
    def sign(self, msg: bytes) -> bytes: ...
    def verify(self, msg: bytes, sig: bytes) -> bool: ...
    def is_public(self) -> bool: ...
    def public_key(self) -> Self: ...
    def to_pem(self, pem_format: str = "PKCS8") -> bytes: ...
    def to_dict(self) -> dict[str, Any]: ...
    def wrap_key(self, key_data: bytes) -> bytes: ...
    def unwrap_key(self, wrapped_key: bytes) -> bytes: ...



from _typeshed import Incomplete

from .base import Client as Client

class LegacyApplicationClient(Client):
    grant_type: str
    def __init__(self, client_id, **kwargs) -> None: ...
    def prepare_request_body(
        self, username, password, body: str = "", scope: Incomplete | None = None, include_client_id: bool = False, **kwargs
    ): ...



from collections.abc import Mapping
from fractions import Fraction
from typing import Any, TypeVar, overload
from typing_extensions import Self

_T = TypeVar("_T")

@overload
def ord_(dta: str) -> int: ...  # type: ignore[overload-overlap]
@overload
def ord_(dta: _T) -> _T: ...
def make_string(seq: str | list[int]) -> str: ...
def make_string_uc(seq: str | list[int]) -> str: ...
def get_gps_coords(tags: Mapping[str, Any]) -> tuple[float, float]: ...

class Ratio(Fraction):
    def __new__(cls, numerator: int = 0, denominator: int | None = None) -> Self: ...
    @property
    def num(self) -> int: ...
    @property
    def den(self) -> int: ...
    def decimal(self) -> float: ...



from _typeshed import Incomplete

from hvac.api.vault_api_base import VaultApiBase

SUPPORTED_MFA_TYPES: Incomplete
SUPPORTED_AUTH_METHODS: Incomplete

class LegacyMfa(VaultApiBase):
    def configure(self, mount_point, mfa_type: str = "duo", force: bool = False): ...
    def read_configuration(self, mount_point): ...
    def configure_duo_access(self, mount_point, host, integration_key, secret_key): ...
    def configure_duo_behavior(
        self, mount_point, push_info: Incomplete | None = None, user_agent: Incomplete | None = None, username_format: str = "%s"
    ): ...
    def read_duo_behavior_configuration(self, mount_point): ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol Buffers - Google's data interchange format
Copyright 2008 Google Inc.  All rights reserved.
https://developers.google.com/protocol-buffers/

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above
copyright notice, this list of conditions and the following disclaimer
in the documentation and/or other materials provided with the
distribution.
    * Neither the name of Google Inc. nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
import builtins
import typing as typing_extensions

import google.protobuf.descriptor
import google.protobuf.internal.well_known_types
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Timestamp(google.protobuf.message.Message, google.protobuf.internal.well_known_types.Timestamp):
    """A Timestamp represents a point in time independent of any time zone or local
    calendar, encoded as a count of seconds and fractions of seconds at
    nanosecond resolution. The count is relative to an epoch at UTC midnight on
    January 1, 1970, in the proleptic Gregorian calendar which extends the
    Gregorian calendar backwards to year one.

    All minutes are 60 seconds long. Leap seconds are "smeared" so that no leap
    second table is needed for interpretation, using a [24-hour linear
    smear](https://developers.google.com/time/smear).

    The range is from 0001-01-01T00:00:00Z to 9999-12-31T23:59:59.999999999Z. By
    restricting to that range, we ensure that we can convert to and from [RFC
    3339](https://www.ietf.org/rfc/rfc3339.txt) date strings.

    # Examples

    Example 1: Compute Timestamp from POSIX `time()`.

        Timestamp timestamp;
        timestamp.set_seconds(time(NULL));
        timestamp.set_nanos(0);

    Example 2: Compute Timestamp from POSIX `gettimeofday()`.

        struct timeval tv;
        gettimeofday(&tv, NULL);

        Timestamp timestamp;
        timestamp.set_seconds(tv.tv_sec);
        timestamp.set_nanos(tv.tv_usec * 1000);

    Example 3: Compute Timestamp from Win32 `GetSystemTimeAsFileTime()`.

        FILETIME ft;
        GetSystemTimeAsFileTime(&ft);
        UINT64 ticks = (((UINT64)ft.dwHighDateTime) << 32) | ft.dwLowDateTime;

        // A Windows tick is 100 nanoseconds. Windows epoch 1601-01-01T00:00:00Z
        // is 11644473600 seconds before Unix epoch 1970-01-01T00:00:00Z.
        Timestamp timestamp;
        timestamp.set_seconds((INT64) ((ticks / 10000000) - 11644473600LL));
        timestamp.set_nanos((INT32) ((ticks % 10000000) * 100));

    Example 4: Compute Timestamp from Java `System.currentTimeMillis()`.

        long millis = System.currentTimeMillis();

        Timestamp timestamp = Timestamp.newBuilder().setSeconds(millis / 1000)
            .setNanos((int) ((millis % 1000) * 1000000)).build();


    Example 5: Compute Timestamp from Java `Instant.now()`.

        Instant now = Instant.now();

        Timestamp timestamp =
            Timestamp.newBuilder().setSeconds(now.getEpochSecond())
                .setNanos(now.getNano()).build();


    Example 6: Compute Timestamp from current time in Python.

        timestamp = Timestamp()
        timestamp.GetCurrentTime()

    # JSON Mapping

    In JSON format, the Timestamp type is encoded as a string in the
    [RFC 3339](https://www.ietf.org/rfc/rfc3339.txt) format. That is, the
    format is "{year}-{month}-{day}T{hour}:{min}:{sec}[.{frac_sec}]Z"
    where {year} is always expressed using four digits while {month}, {day},
    {hour}, {min}, and {sec} are zero-padded to two digits each. The fractional
    seconds, which can go up to 9 digits (i.e. up to 1 nanosecond resolution),
    are optional. The "Z" suffix indicates the timezone ("UTC"); the timezone
    is required. A proto3 JSON serializer should always use UTC (as indicated by
    "Z") when printing the Timestamp type and a proto3 JSON parser should be
    able to accept both UTC and other timezones (as indicated by an offset).

    For example, "2017-01-15T01:30:15.01Z" encodes 15.01 seconds past
    01:30 UTC on January 15, 2017.

    In JavaScript, one can convert a Date object to this format using the
    standard
    [toISOString()](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString)
    method. In Python, a standard `datetime.datetime` object can be converted
    to this format using
    [`strftime`](https://docs.python.org/2/library/time.html#time.strftime) with
    the time format spec '%Y-%m-%dT%H:%M:%S.%fZ'. Likewise, in Java, one can use
    the Joda Time's [`ISODateTimeFormat.dateTime()`](
    http://www.joda.org/joda-time/apidocs/org/joda/time/format/ISODateTimeFormat.html#dateTime%2D%2D
    ) to obtain a formatter capable of generating timestamps in this format.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECONDS_FIELD_NUMBER: builtins.int
    NANOS_FIELD_NUMBER: builtins.int
    seconds: builtins.int
    """Represents seconds of UTC time since Unix epoch
    1970-01-01T00:00:00Z. Must be from 0001-01-01T00:00:00Z to
    9999-12-31T23:59:59Z inclusive.
    """
    nanos: builtins.int
    """Non-negative fractions of a second at nanosecond resolution. Negative
    second values with fractions must still have non-negative nanos values
    that count forward in time. Must be from 0 to 999,999,999
    inclusive.
    """
    def __init__(
        self,
        *,
        seconds: builtins.int | None = ...,
        nanos: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["nanos", b"nanos", "seconds", b"seconds"]) -> None: ...

global___Timestamp = Timestamp



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def metric_closure(G, weight: str = "weight"): ...
@_dispatch
def steiner_tree(G, terminal_nodes, weight: str = "weight", method: Incomplete | None = None): ...



from _typeshed import Incomplete
from typing import NoReturn

from stripe import error as error
from stripe.api_resources.abstract import (
    DeletableAPIResource as DeletableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
)
from stripe.api_resources.account import Account as Account
from stripe.api_resources.customer import Customer as Customer
from stripe.api_resources.recipient import Recipient as Recipient

class Card(DeletableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str
    def instance_url(self) -> str: ...
    @classmethod
    def modify(cls, sid, **params) -> NoReturn: ...
    @classmethod
    def retrieve(
        cls,
        id,
        api_key: Incomplete | None = None,
        stripe_version: Incomplete | None = None,
        stripe_account: Incomplete | None = None,
        **params,
    ) -> NoReturn: ...



from _typeshed import Incomplete
from collections.abc import Generator, Iterator
from typing import Any

from influxdb_client import Script, ScriptCreateRequest, ScriptUpdateRequest
from influxdb_client.client._base import _BaseQueryApi
from influxdb_client.client.flux_table import CSVIterator, FluxRecord, TableList

class InvokableScriptsApi(_BaseQueryApi):
    def __init__(self, influxdb_client) -> None: ...
    def create_script(self, create_request: ScriptCreateRequest) -> Script: ...
    def update_script(self, script_id: str, update_request: ScriptUpdateRequest) -> Script: ...
    def delete_script(self, script_id: str) -> None: ...
    def find_scripts(self, **kwargs): ...
    def invoke_script(self, script_id: str, params: dict[Incomplete, Incomplete] | None = None) -> TableList: ...
    def invoke_script_stream(
        self, script_id: str, params: dict[Incomplete, Incomplete] | None = None
    ) -> Generator[FluxRecord, Any, None]: ...
    def invoke_script_data_frame(
        self, script_id: str, params: dict[Incomplete, Incomplete] | None = None, data_frame_index: list[str] | None = None
    ): ...
    def invoke_script_data_frame_stream(
        self, script_id: str, params: dict[Incomplete, Incomplete] | None = None, data_frame_index: list[str] | None = None
    ): ...
    def invoke_script_csv(self, script_id: str, params: dict[Incomplete, Incomplete] | None = None) -> CSVIterator: ...
    def invoke_script_raw(self, script_id: str, params: dict[Incomplete, Incomplete] | None = None) -> Iterator[list[str]]: ...



import sys
from typing import IO, Any, Literal, overload
from typing_extensions import deprecated

__all__ = [
    "get_config_h_filename",
    "get_config_var",
    "get_config_vars",
    "get_makefile_filename",
    "get_path",
    "get_path_names",
    "get_paths",
    "get_platform",
    "get_python_version",
    "get_scheme_names",
    "parse_config_h",
]

@overload
@deprecated("SO is deprecated, use EXT_SUFFIX. Support is removed in Python 3.11")
def get_config_var(name: Literal["SO"]) -> Any: ...
@overload
def get_config_var(name: str) -> Any: ...
@overload
def get_config_vars() -> dict[str, Any]: ...
@overload
def get_config_vars(arg: str, /, *args: str) -> list[Any]: ...
def get_scheme_names() -> tuple[str, ...]: ...

if sys.version_info >= (3, 10):
    def get_default_scheme() -> str: ...
    def get_preferred_scheme(key: Literal["prefix", "home", "user"]) -> str: ...

def get_path_names() -> tuple[str, ...]: ...
def get_path(name: str, scheme: str = ..., vars: dict[str, Any] | None = None, expand: bool = True) -> str: ...
def get_paths(scheme: str = ..., vars: dict[str, Any] | None = None, expand: bool = True) -> dict[str, str]: ...
def get_python_version() -> str: ...
def get_platform() -> str: ...

if sys.version_info >= (3, 11):
    def is_python_build(check_home: object = None) -> bool: ...

else:
    def is_python_build(check_home: bool = False) -> bool: ...

def parse_config_h(fp: IO[Any], vars: dict[str, Any] | None = None) -> dict[str, Any]: ...
def get_config_h_filename() -> str: ...
def get_makefile_filename() -> str: ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import WesternCalendar

class Spain(WesternCalendar):
    include_epiphany: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    include_assumption: ClassVar[bool]
    include_all_saints: ClassVar[bool]
    include_immaculate_conception: ClassVar[bool]
    include_labour_day: ClassVar[bool]
    labour_day_label: ClassVar[str]
    FIXED_HOLIDAYS: Incomplete

class Andalusia(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class Aragon(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class CastileAndLeon(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class CastillaLaMancha(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class CanaryIslands(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class Catalonia(Spain):
    include_easter_monday: ClassVar[bool]
    include_boxing_day: ClassVar[bool]
    boxing_day_label: ClassVar[str]
    FIXED_HOLIDAYS: Incomplete

class Extremadura(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class Galicia(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class BalearicIslands(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]
    include_easter_monday: ClassVar[bool]

class LaRioja(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class CommunityofMadrid(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class Murcia(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class Navarre(Spain):
    include_holy_thursday: ClassVar[bool]
    include_easter_monday: ClassVar[bool]

class Asturias(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class BasqueCountry(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]
    include_easter_monday: ClassVar[bool]

class Cantabria(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_holy_thursday: ClassVar[bool]

class ValencianCommunity(Spain):
    FIXED_HOLIDAYS: Incomplete
    include_easter_monday: ClassVar[bool]



from _typeshed import SupportsRichComparison
from collections.abc import Callable
from typing import IO, Any, TypeVar, overload
from typing_extensions import TypeAlias

from simplejson.decoder import JSONDecoder as JSONDecoder
from simplejson.encoder import JSONEncoder as JSONEncoder, JSONEncoderForHTML as JSONEncoderForHTML
from simplejson.raw_json import RawJSON as RawJSON
from simplejson.scanner import JSONDecodeError as JSONDecodeError

_LoadsString: TypeAlias = str | bytes | bytearray
_T = TypeVar("_T")

@overload
def dumps(
    obj: Any,
    skipkeys: bool = ...,
    ensure_ascii: bool = ...,
    check_circular: bool = ...,
    allow_nan: bool = ...,
    *,
    cls: type[JSONEncoder],
    indent: str | int | None = ...,
    separators: tuple[str, str] | None = ...,
    encoding: str = ...,
    default: Callable[[Any], Any] | None = ...,
    use_decimal: bool = ...,
    namedtuple_as_object: bool = ...,
    tuple_as_array: bool = ...,
    bigint_as_string: bool = ...,
    sort_keys: bool = ...,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = ...,
    for_json: bool = ...,
    ignore_nan: bool = ...,
    int_as_string_bitcount: int | None = ...,
    iterable_as_array: bool = ...,
    **kw: Any,
) -> str: ...
@overload
def dumps(
    obj: Any,
    skipkeys: bool = ...,
    ensure_ascii: bool = ...,
    check_circular: bool = ...,
    allow_nan: bool = ...,
    cls: type[JSONEncoder] | None = ...,
    indent: str | int | None = ...,
    separators: tuple[str, str] | None = ...,
    encoding: str = ...,
    default: Callable[[Any], Any] | None = ...,
    use_decimal: bool = ...,
    namedtuple_as_object: bool = ...,
    tuple_as_array: bool = ...,
    bigint_as_string: bool = ...,
    sort_keys: bool = ...,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = ...,
    for_json: bool = ...,
    ignore_nan: bool = ...,
    int_as_string_bitcount: int | None = ...,
    iterable_as_array: bool = ...,
) -> str: ...
@overload
def dump(
    obj: Any,
    fp: IO[str],
    skipkeys: bool = ...,
    ensure_ascii: bool = ...,
    check_circular: bool = ...,
    allow_nan: bool = ...,
    *,
    cls: type[JSONEncoder],
    indent: str | int | None = ...,
    separators: tuple[str, str] | None = ...,
    encoding: str = ...,
    default: Callable[[Any], Any] | None = ...,
    use_decimal: bool = ...,
    namedtuple_as_object: bool = ...,
    tuple_as_array: bool = ...,
    bigint_as_string: bool = ...,
    sort_keys: bool = ...,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = ...,
    for_json: bool = ...,
    ignore_nan: bool = ...,
    int_as_string_bitcount: int | None = ...,
    iterable_as_array: bool = ...,
    **kw: Any,
) -> None: ...
@overload
def dump(
    obj: Any,
    fp: IO[str],
    skipkeys: bool = ...,
    ensure_ascii: bool = ...,
    check_circular: bool = ...,
    allow_nan: bool = ...,
    cls: type[JSONEncoder] | None = ...,
    indent: str | int | None = ...,
    separators: tuple[str, str] | None = ...,
    encoding: str = ...,
    default: Callable[[Any], Any] | None = ...,
    use_decimal: bool = ...,
    namedtuple_as_object: bool = ...,
    tuple_as_array: bool = ...,
    bigint_as_string: bool = ...,
    sort_keys: bool = ...,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = ...,
    for_json: bool = ...,
    ignore_nan: bool = ...,
    int_as_string_bitcount: int | None = ...,
    iterable_as_array: bool = ...,
) -> None: ...
@overload
def loads(
    s: _LoadsString,
    encoding: str | None = ...,
    *,
    cls: type[JSONDecoder],
    object_hook: Callable[[dict[Any, Any]], Any] | None = ...,
    parse_float: Callable[[str], Any] | None = ...,
    parse_int: Callable[[str], Any] | None = ...,
    parse_constant: Callable[[str], Any] | None = ...,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = ...,
    use_decimal: bool = ...,
    allow_nan: bool = ...,
    **kw: Any,
) -> Any: ...
@overload
def loads(
    s: _LoadsString,
    encoding: str | None = ...,
    cls: type[JSONDecoder] | None = ...,
    object_hook: Callable[[dict[Any, Any]], Any] | None = ...,
    parse_float: Callable[[str], Any] | None = ...,
    parse_int: Callable[[str], Any] | None = ...,
    parse_constant: Callable[[str], Any] | None = ...,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = ...,
    use_decimal: bool = ...,
    allow_nan: bool = ...,
) -> Any: ...
@overload
def load(
    fp: IO[str],
    encoding: str | None = ...,
    *,
    cls: type[JSONDecoder],
    object_hook: Callable[[dict[Any, Any]], Any] | None = ...,
    parse_float: Callable[[str], Any] | None = ...,
    parse_int: Callable[[str], Any] | None = ...,
    parse_constant: Callable[[str], Any] | None = ...,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = ...,
    use_decimal: bool = ...,
    allow_nan: bool = ...,
    **kw: Any,
) -> Any: ...
@overload
def load(
    fp: IO[str],
    encoding: str | None = ...,
    cls: type[JSONDecoder] | None = ...,
    object_hook: Callable[[dict[Any, Any]], Any] | None = ...,
    parse_float: Callable[[str], Any] | None = ...,
    parse_int: Callable[[str], Any] | None = ...,
    parse_constant: Callable[[str], Any] | None = ...,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = ...,
    use_decimal: bool = ...,
    allow_nan: bool = ...,
) -> Any: ...
def simple_first(kv: tuple[_T, object]) -> tuple[bool, _T]: ...



import email.message
import io
import sys
from _typeshed import StrPath, SupportsNoArgReadline, SupportsRead
from abc import ABCMeta, abstractmethod
from collections.abc import Callable, Iterable, Iterator, Mapping, Sequence
from types import TracebackType
from typing import IO, Any, AnyStr, Generic, Literal, Protocol, TypeVar, overload
from typing_extensions import Self, TypeAlias

if sys.version_info >= (3, 9):
    from types import GenericAlias

__all__ = [
    "Mailbox",
    "Maildir",
    "mbox",
    "MH",
    "Babyl",
    "MMDF",
    "Message",
    "MaildirMessage",
    "mboxMessage",
    "MHMessage",
    "BabylMessage",
    "MMDFMessage",
    "Error",
    "NoSuchMailboxError",
    "NotEmptyError",
    "ExternalClashError",
    "FormatError",
]

_T = TypeVar("_T")
_MessageT = TypeVar("_MessageT", bound=Message)

class _SupportsReadAndReadline(SupportsRead[bytes], SupportsNoArgReadline[bytes], Protocol): ...

_MessageData: TypeAlias = email.message.Message | bytes | str | io.StringIO | _SupportsReadAndReadline

class _HasIteritems(Protocol):
    def iteritems(self) -> Iterator[tuple[str, _MessageData]]: ...

class _HasItems(Protocol):
    def items(self) -> Iterator[tuple[str, _MessageData]]: ...

linesep: bytes

class Mailbox(Generic[_MessageT]):
    _path: str  # undocumented
    _factory: Callable[[IO[Any]], _MessageT] | None  # undocumented
    @overload
    def __init__(self, path: StrPath, factory: Callable[[IO[Any]], _MessageT], create: bool = True) -> None: ...
    @overload
    def __init__(self, path: StrPath, factory: None = None, create: bool = True) -> None: ...
    @abstractmethod
    def add(self, message: _MessageData) -> str: ...
    @abstractmethod
    def remove(self, key: str) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def discard(self, key: str) -> None: ...
    @abstractmethod
    def __setitem__(self, key: str, message: _MessageData) -> None: ...
    @overload
    def get(self, key: str, default: None = None) -> _MessageT | None: ...
    @overload
    def get(self, key: str, default: _T) -> _MessageT | _T: ...
    def __getitem__(self, key: str) -> _MessageT: ...
    @abstractmethod
    def get_message(self, key: str) -> _MessageT: ...
    def get_string(self, key: str) -> str: ...
    @abstractmethod
    def get_bytes(self, key: str) -> bytes: ...
    # As '_ProxyFile' doesn't implement the full IO spec, and BytesIO is incompatible with it, get_file return is Any here
    @abstractmethod
    def get_file(self, key: str) -> Any: ...
    @abstractmethod
    def iterkeys(self) -> Iterator[str]: ...
    def keys(self) -> list[str]: ...
    def itervalues(self) -> Iterator[_MessageT]: ...
    def __iter__(self) -> Iterator[_MessageT]: ...
    def values(self) -> list[_MessageT]: ...
    def iteritems(self) -> Iterator[tuple[str, _MessageT]]: ...
    def items(self) -> list[tuple[str, _MessageT]]: ...
    @abstractmethod
    def __contains__(self, key: str) -> bool: ...
    @abstractmethod
    def __len__(self) -> int: ...
    def clear(self) -> None: ...
    @overload
    def pop(self, key: str, default: None = None) -> _MessageT | None: ...
    @overload
    def pop(self, key: str, default: _T) -> _MessageT | _T: ...
    def popitem(self) -> tuple[str, _MessageT]: ...
    def update(self, arg: _HasIteritems | _HasItems | Iterable[tuple[str, _MessageData]] | None = None) -> None: ...
    @abstractmethod
    def flush(self) -> None: ...
    @abstractmethod
    def lock(self) -> None: ...
    @abstractmethod
    def unlock(self) -> None: ...
    @abstractmethod
    def close(self) -> None: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class Maildir(Mailbox[MaildirMessage]):
    colon: str
    def __init__(
        self, dirname: StrPath, factory: Callable[[IO[Any]], MaildirMessage] | None = None, create: bool = True
    ) -> None: ...
    def add(self, message: _MessageData) -> str: ...
    def remove(self, key: str) -> None: ...
    def __setitem__(self, key: str, message: _MessageData) -> None: ...
    def get_message(self, key: str) -> MaildirMessage: ...
    def get_bytes(self, key: str) -> bytes: ...
    def get_file(self, key: str) -> _ProxyFile[bytes]: ...
    def iterkeys(self) -> Iterator[str]: ...
    def __contains__(self, key: str) -> bool: ...
    def __len__(self) -> int: ...
    def flush(self) -> None: ...
    def lock(self) -> None: ...
    def unlock(self) -> None: ...
    def close(self) -> None: ...
    def list_folders(self) -> list[str]: ...
    def get_folder(self, folder: str) -> Maildir: ...
    def add_folder(self, folder: str) -> Maildir: ...
    def remove_folder(self, folder: str) -> None: ...
    def clean(self) -> None: ...
    def next(self) -> str | None: ...

class _singlefileMailbox(Mailbox[_MessageT], metaclass=ABCMeta):
    def add(self, message: _MessageData) -> str: ...
    def remove(self, key: str) -> None: ...
    def __setitem__(self, key: str, message: _MessageData) -> None: ...
    def iterkeys(self) -> Iterator[str]: ...
    def __contains__(self, key: str) -> bool: ...
    def __len__(self) -> int: ...
    def lock(self) -> None: ...
    def unlock(self) -> None: ...
    def flush(self) -> None: ...
    def close(self) -> None: ...

class _mboxMMDF(_singlefileMailbox[_MessageT]):
    def get_message(self, key: str) -> _MessageT: ...
    def get_file(self, key: str, from_: bool = False) -> _PartialFile[bytes]: ...
    def get_bytes(self, key: str, from_: bool = False) -> bytes: ...
    def get_string(self, key: str, from_: bool = False) -> str: ...

class mbox(_mboxMMDF[mboxMessage]):
    def __init__(self, path: StrPath, factory: Callable[[IO[Any]], mboxMessage] | None = None, create: bool = True) -> None: ...

class MMDF(_mboxMMDF[MMDFMessage]):
    def __init__(self, path: StrPath, factory: Callable[[IO[Any]], MMDFMessage] | None = None, create: bool = True) -> None: ...

class MH(Mailbox[MHMessage]):
    def __init__(self, path: StrPath, factory: Callable[[IO[Any]], MHMessage] | None = None, create: bool = True) -> None: ...
    def add(self, message: _MessageData) -> str: ...
    def remove(self, key: str) -> None: ...
    def __setitem__(self, key: str, message: _MessageData) -> None: ...
    def get_message(self, key: str) -> MHMessage: ...
    def get_bytes(self, key: str) -> bytes: ...
    def get_file(self, key: str) -> _ProxyFile[bytes]: ...
    def iterkeys(self) -> Iterator[str]: ...
    def __contains__(self, key: str) -> bool: ...
    def __len__(self) -> int: ...
    def flush(self) -> None: ...
    def lock(self) -> None: ...
    def unlock(self) -> None: ...
    def close(self) -> None: ...
    def list_folders(self) -> list[str]: ...
    def get_folder(self, folder: StrPath) -> MH: ...
    def add_folder(self, folder: StrPath) -> MH: ...
    def remove_folder(self, folder: StrPath) -> None: ...
    def get_sequences(self) -> dict[str, list[int]]: ...
    def set_sequences(self, sequences: Mapping[str, Sequence[int]]) -> None: ...
    def pack(self) -> None: ...

class Babyl(_singlefileMailbox[BabylMessage]):
    def __init__(self, path: StrPath, factory: Callable[[IO[Any]], BabylMessage] | None = None, create: bool = True) -> None: ...
    def get_message(self, key: str) -> BabylMessage: ...
    def get_bytes(self, key: str) -> bytes: ...
    def get_file(self, key: str) -> IO[bytes]: ...
    def get_labels(self) -> list[str]: ...

class Message(email.message.Message):
    def __init__(self, message: _MessageData | None = None) -> None: ...

class MaildirMessage(Message):
    def get_subdir(self) -> str: ...
    def set_subdir(self, subdir: Literal["new", "cur"]) -> None: ...
    def get_flags(self) -> str: ...
    def set_flags(self, flags: Iterable[str]) -> None: ...
    def add_flag(self, flag: str) -> None: ...
    def remove_flag(self, flag: str) -> None: ...
    def get_date(self) -> int: ...
    def set_date(self, date: float) -> None: ...
    def get_info(self) -> str: ...
    def set_info(self, info: str) -> None: ...

class _mboxMMDFMessage(Message):
    def get_from(self) -> str: ...
    def set_from(self, from_: str, time_: bool | tuple[int, int, int, int, int, int, int, int, int] | None = None) -> None: ...
    def get_flags(self) -> str: ...
    def set_flags(self, flags: Iterable[str]) -> None: ...
    def add_flag(self, flag: str) -> None: ...
    def remove_flag(self, flag: str) -> None: ...

class mboxMessage(_mboxMMDFMessage): ...

class MHMessage(Message):
    def get_sequences(self) -> list[str]: ...
    def set_sequences(self, sequences: Iterable[str]) -> None: ...
    def add_sequence(self, sequence: str) -> None: ...
    def remove_sequence(self, sequence: str) -> None: ...

class BabylMessage(Message):
    def get_labels(self) -> list[str]: ...
    def set_labels(self, labels: Iterable[str]) -> None: ...
    def add_label(self, label: str) -> None: ...
    def remove_label(self, label: str) -> None: ...
    def get_visible(self) -> Message: ...
    def set_visible(self, visible: _MessageData) -> None: ...
    def update_visible(self) -> None: ...

class MMDFMessage(_mboxMMDFMessage): ...

class _ProxyFile(Generic[AnyStr]):
    def __init__(self, f: IO[AnyStr], pos: int | None = None) -> None: ...
    def read(self, size: int | None = None) -> AnyStr: ...
    def read1(self, size: int | None = None) -> AnyStr: ...
    def readline(self, size: int | None = None) -> AnyStr: ...
    def readlines(self, sizehint: int | None = None) -> list[AnyStr]: ...
    def __iter__(self) -> Iterator[AnyStr]: ...
    def tell(self) -> int: ...
    def seek(self, offset: int, whence: int = 0) -> None: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, exc_type: type[BaseException] | None, exc: BaseException | None, tb: TracebackType | None) -> None: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def seekable(self) -> bool: ...
    def flush(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class _PartialFile(_ProxyFile[AnyStr]):
    def __init__(self, f: IO[AnyStr], start: int | None = None, stop: int | None = None) -> None: ...

class Error(Exception): ...
class NoSuchMailboxError(Error): ...
class NotEmptyError(Error): ...
class ExternalClashError(Error): ...
class FormatError(Error): ...



import re
import sys
from collections.abc import Generator

if sys.platform == "win32":
    from serial.tools.list_ports_windows import comports as comports
else:
    from serial.tools.list_ports_posix import comports as comports

def grep(regexp: str | re.Pattern[str], include_links: bool = False) -> Generator[tuple[str, str, str], None, None]: ...
def main() -> None: ...



from _typeshed import Incomplete

def prepare_grant_uri(
    uri,
    client_id,
    response_type,
    redirect_uri: Incomplete | None = None,
    scope: Incomplete | None = None,
    state: Incomplete | None = None,
    code_challenge: str | None = None,
    code_challenge_method: str | None = "plain",
    **kwargs,
): ...
def prepare_token_request(
    grant_type, body: str = "", include_client_id: bool = True, code_verifier: str | None = None, **kwargs
): ...
def prepare_token_revocation_request(
    url, token, token_type_hint: str = "access_token", callback: Incomplete | None = None, body: str = "", **kwargs
): ...
def parse_authorization_code_response(uri, state: Incomplete | None = None): ...
def parse_implicit_response(uri, state: Incomplete | None = None, scope: Incomplete | None = None): ...
def parse_token_response(body, scope: Incomplete | None = None): ...
def validate_token_parameters(params) -> None: ...



from pyasn1.codec.ber.decoder import AbstractPayloadDecoder
from pyasn1.codec.cer import decoder
from pyasn1.type.tag import TagSet

__all__ = ["decode", "StreamingDecoder"]

class BitStringPayloadDecoder(decoder.BitStringPayloadDecoder):
    supportConstructedForm: bool

class OctetStringPayloadDecoder(decoder.OctetStringPayloadDecoder):
    supportConstructedForm: bool

RealPayloadDecoder = decoder.RealPayloadDecoder

TAG_MAP: dict[TagSet, AbstractPayloadDecoder]
TYPE_MAP: dict[int, AbstractPayloadDecoder]
# deprecated aliases
tagMap = TAG_MAP
typeMap = TYPE_MAP

class SingleItemDecoder(decoder.SingleItemDecoder):
    TAG_MAP: dict[TagSet, AbstractPayloadDecoder]
    TYPE_MAP: dict[int, AbstractPayloadDecoder]

    supportIndefLength: bool

class StreamingDecoder(decoder.StreamingDecoder):
    SINGLE_ITEM_DECODER: type[SingleItemDecoder]

class Decoder(decoder.Decoder):
    STREAMING_DECODER: type[StreamingDecoder]

decode: Decoder



import sys

if sys.platform != "win32":
    from collections.abc import Sequence
    from typing import TypedDict

    from gevent._types import _Watcher
    from gevent.hub import Hub
    from gevent.resolver import AbstractResolver
    from gevent.resolver.cares import channel

    class _ChannelArgs(TypedDict):
        flags: str | int | None
        timeout: str | float | None
        tries: str | int | None
        ndots: str | int | None
        udp_port: str | int | None
        tcp_port: str | int | None
        servers: Sequence[str] | str | None

    class Resolver(AbstractResolver):
        cares_class: type[channel]
        hub: Hub
        cares: channel
        pid: int
        params: _ChannelArgs
        fork_watcher: _Watcher
        def __init__(
            self,
            hub: Hub | None = None,
            use_environ: bool = True,
            *,
            flags: str | int | None = None,
            timeout: str | float | None = None,
            tries: str | int | None = None,
            ndots: str | int | None = None,
            udp_port: str | int | None = None,
            tcp_port: str | int | None = None,
            servers: Sequence[str] | str | None = None,
        ) -> None: ...
        def __del__(self) -> None: ...



import socket
from _typeshed import ReadableBuffer, SliceableBuffer

DEFAULT_TIMEOUT: int
DEFAULT_MAXSIZE: int

class BufferedSocket:
    sock: socket.socket
    rbuf: bytes
    sbuf: list[SliceableBuffer]
    maxsize: int
    timeout: int
    def __init__(self, sock: socket.socket, timeout: int = ..., maxsize: int = 32768, recvsize: int = ...) -> None: ...
    def settimeout(self, timeout: float) -> None: ...
    def gettimeout(self) -> float: ...
    def setblocking(self, blocking: bool) -> None: ...
    def setmaxsize(self, maxsize) -> None: ...
    def getrecvbuffer(self) -> bytes: ...
    def getsendbuffer(self) -> bytes: ...
    def recv(self, size: int, flags: int = 0, timeout: float = ...) -> bytes: ...
    def peek(self, size: int, timeout: float = ...) -> bytes: ...
    def recv_close(self, timeout: float = ..., maxsize: int = ...) -> bytes: ...
    def recv_until(
        self, delimiter: ReadableBuffer, timeout: float = ..., maxsize: int = ..., with_delimiter: bool = False
    ) -> bytes: ...
    def recv_size(self, size: int, timeout: float = ...) -> bytes: ...
    def send(self, data: SliceableBuffer, flags: int = 0, timeout: float = ...) -> str: ...
    def sendall(self, data: SliceableBuffer, flags: int = 0, timeout: float = ...) -> str: ...
    def flush(self) -> None: ...
    def buffer(self, data: SliceableBuffer) -> None: ...
    def getsockname(self) -> str: ...
    def getpeername(self) -> str: ...
    def getsockopt(self, level: int, optname: int, buflen: int | None = None) -> bytes | int: ...
    def setsockopt(self, level: int, optname: int, value: int | ReadableBuffer | None) -> bytes | int: ...
    @property
    def type(self) -> int: ...
    @property
    def family(self) -> int: ...
    @property
    def proto(self) -> int: ...
    def fileno(self) -> int: ...
    rbuf_unconsumed: bytes
    def close(self) -> None: ...
    def shutdown(self, how: int) -> None: ...

class Error(socket.error): ...
class ConnectionClosed(Error): ...

class MessageTooLong(Error):
    def __init__(self, bytes_read: int | None = None, delimiter: str | None = None) -> None: ...

class Timeout(socket.timeout, Error):
    def __init__(self, timeout: float, extra: str = "") -> None: ...

class NetstringSocket:
    bsock: BufferedSocket
    timeout: float
    maxsize: int
    def __init__(self, sock: socket.socket, timeout: float = 10, maxsize: int = 32768) -> None: ...
    def fileno(self) -> int: ...
    def settimeout(self, timeout: float) -> None: ...
    def setmaxsize(self, maxsize: int) -> None: ...
    def read_ns(self, timeout: float = ..., maxsize: int = ...): ...
    def write_ns(self, payload: bytes) -> None: ...

class NetstringProtocolError(Error): ...

class NetstringInvalidSize(NetstringProtocolError):
    def __init__(self, msg: str) -> None: ...

class NetstringMessageTooLong(NetstringProtocolError):
    def __init__(self, size: int, maxsize: int) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar

from .core import UnitedStates

class Guam(UnitedStates):
    FIXED_HOLIDAYS: Incomplete
    include_all_souls: ClassVar[bool]
    include_immaculate_conception: ClassVar[bool]
    immaculate_conception_label: ClassVar[str]



from collections.abc import Generator, Hashable, Iterable

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatch

@_dispatch
def strongly_connected_components(G: Graph[_Node]) -> Generator[set[_Node], None, None]: ...
@_dispatch
def kosaraju_strongly_connected_components(G: Graph[_Node], source: _Node | None = None) -> Generator[set[_Node], None, None]: ...
@_dispatch
def strongly_connected_components_recursive(G: Graph[_Node]) -> Generator[set[_Node], None, None]: ...
@_dispatch
def number_strongly_connected_components(G: Graph[Hashable]) -> int: ...
@_dispatch
def is_strongly_connected(G: Graph[Hashable]) -> bool: ...
@_dispatch
def condensation(G: DiGraph[_Node], scc: Iterable[Iterable[_Node]] | None = None) -> DiGraph[int]: ...



import sys
from _typeshed import SupportsGetItem
from collections.abc import Callable, Container, Iterable, MutableMapping, MutableSequence, Sequence
from typing import Any, AnyStr, Generic, Protocol, SupportsAbs, SupportsIndex, TypeVar, final, overload
from typing_extensions import ParamSpec, TypeAlias, TypeVarTuple, Unpack

_R = TypeVar("_R")
_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)
_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2")
_K = TypeVar("_K")
_V = TypeVar("_V")
_P = ParamSpec("_P")
_Ts = TypeVarTuple("_Ts")

# The following protocols return "Any" instead of bool, since the comparison
# operators can be overloaded to return an arbitrary object. For example,
# the numpy.array comparison dunders return another numpy.array.

class _SupportsDunderLT(Protocol):
    def __lt__(self, other: Any, /) -> Any: ...

class _SupportsDunderGT(Protocol):
    def __gt__(self, other: Any, /) -> Any: ...

class _SupportsDunderLE(Protocol):
    def __le__(self, other: Any, /) -> Any: ...

class _SupportsDunderGE(Protocol):
    def __ge__(self, other: Any, /) -> Any: ...

_SupportsComparison: TypeAlias = _SupportsDunderLE | _SupportsDunderGE | _SupportsDunderGT | _SupportsDunderLT

class _SupportsInversion(Protocol[_T_co]):
    def __invert__(self) -> _T_co: ...

class _SupportsNeg(Protocol[_T_co]):
    def __neg__(self) -> _T_co: ...

class _SupportsPos(Protocol[_T_co]):
    def __pos__(self) -> _T_co: ...

# All four comparison functions must have the same signature, or we get false-positive errors
def lt(a: _SupportsComparison, b: _SupportsComparison, /) -> Any: ...
def le(a: _SupportsComparison, b: _SupportsComparison, /) -> Any: ...
def eq(a: object, b: object, /) -> Any: ...
def ne(a: object, b: object, /) -> Any: ...
def ge(a: _SupportsComparison, b: _SupportsComparison, /) -> Any: ...
def gt(a: _SupportsComparison, b: _SupportsComparison, /) -> Any: ...
def not_(a: object, /) -> bool: ...
def truth(a: object, /) -> bool: ...
def is_(a: object, b: object, /) -> bool: ...
def is_not(a: object, b: object, /) -> bool: ...
def abs(a: SupportsAbs[_T], /) -> _T: ...
def add(a: Any, b: Any, /) -> Any: ...
def and_(a: Any, b: Any, /) -> Any: ...
def floordiv(a: Any, b: Any, /) -> Any: ...
def index(a: SupportsIndex, /) -> int: ...
def inv(a: _SupportsInversion[_T_co], /) -> _T_co: ...
def invert(a: _SupportsInversion[_T_co], /) -> _T_co: ...
def lshift(a: Any, b: Any, /) -> Any: ...
def mod(a: Any, b: Any, /) -> Any: ...
def mul(a: Any, b: Any, /) -> Any: ...
def matmul(a: Any, b: Any, /) -> Any: ...
def neg(a: _SupportsNeg[_T_co], /) -> _T_co: ...
def or_(a: Any, b: Any, /) -> Any: ...
def pos(a: _SupportsPos[_T_co], /) -> _T_co: ...
def pow(a: Any, b: Any, /) -> Any: ...
def rshift(a: Any, b: Any, /) -> Any: ...
def sub(a: Any, b: Any, /) -> Any: ...
def truediv(a: Any, b: Any, /) -> Any: ...
def xor(a: Any, b: Any, /) -> Any: ...
def concat(a: Sequence[_T], b: Sequence[_T], /) -> Sequence[_T]: ...
def contains(a: Container[object], b: object, /) -> bool: ...
def countOf(a: Iterable[object], b: object, /) -> int: ...
@overload
def delitem(a: MutableSequence[Any], b: SupportsIndex, /) -> None: ...
@overload
def delitem(a: MutableSequence[Any], b: slice, /) -> None: ...
@overload
def delitem(a: MutableMapping[_K, Any], b: _K, /) -> None: ...
@overload
def getitem(a: Sequence[_T], b: slice, /) -> Sequence[_T]: ...
@overload
def getitem(a: SupportsGetItem[_K, _V], b: _K, /) -> _V: ...
def indexOf(a: Iterable[_T], b: _T, /) -> int: ...
@overload
def setitem(a: MutableSequence[_T], b: SupportsIndex, c: _T, /) -> None: ...
@overload
def setitem(a: MutableSequence[_T], b: slice, c: Sequence[_T], /) -> None: ...
@overload
def setitem(a: MutableMapping[_K, _V], b: _K, c: _V, /) -> None: ...
def length_hint(obj: object, default: int = 0, /) -> int: ...
@final
class attrgetter(Generic[_T_co]):
    @overload
    def __new__(cls, attr: str, /) -> attrgetter[Any]: ...
    @overload
    def __new__(cls, attr: str, attr2: str, /) -> attrgetter[tuple[Any, Any]]: ...
    @overload
    def __new__(cls, attr: str, attr2: str, attr3: str, /) -> attrgetter[tuple[Any, Any, Any]]: ...
    @overload
    def __new__(cls, attr: str, attr2: str, attr3: str, attr4: str, /) -> attrgetter[tuple[Any, Any, Any, Any]]: ...
    @overload
    def __new__(cls, attr: str, /, *attrs: str) -> attrgetter[tuple[Any, ...]]: ...
    def __call__(self, obj: Any, /) -> _T_co: ...

@final
class itemgetter(Generic[_T_co]):
    @overload
    def __new__(cls, item: _T, /) -> itemgetter[_T]: ...
    @overload
    def __new__(cls, item1: _T1, item2: _T2, /, *items: Unpack[_Ts]) -> itemgetter[tuple[_T1, _T2, Unpack[_Ts]]]: ...
    # __key: _KT_contra in SupportsGetItem seems to be causing variance issues, ie:
    # TypeVar "_KT_contra@SupportsGetItem" is contravariant
    #   "tuple[int, int]" is incompatible with protocol "SupportsIndex"
    # preventing [_T_co, ...] instead of [Any, ...]
    #
    # A suspected mypy issue prevents using [..., _T] instead of [..., Any] here.
    # https://github.com/python/mypy/issues/14032
    def __call__(self, obj: SupportsGetItem[Any, Any]) -> Any: ...

@final
class methodcaller:
    def __init__(self, name: str, /, *args: Any, **kwargs: Any) -> None: ...
    def __call__(self, obj: Any) -> Any: ...

def iadd(a: Any, b: Any, /) -> Any: ...
def iand(a: Any, b: Any, /) -> Any: ...
def iconcat(a: Any, b: Any, /) -> Any: ...
def ifloordiv(a: Any, b: Any, /) -> Any: ...
def ilshift(a: Any, b: Any, /) -> Any: ...
def imod(a: Any, b: Any, /) -> Any: ...
def imul(a: Any, b: Any, /) -> Any: ...
def imatmul(a: Any, b: Any, /) -> Any: ...
def ior(a: Any, b: Any, /) -> Any: ...
def ipow(a: Any, b: Any, /) -> Any: ...
def irshift(a: Any, b: Any, /) -> Any: ...
def isub(a: Any, b: Any, /) -> Any: ...
def itruediv(a: Any, b: Any, /) -> Any: ...
def ixor(a: Any, b: Any, /) -> Any: ...

if sys.version_info >= (3, 11):
    def call(obj: Callable[_P, _R], /, *args: _P.args, **kwargs: _P.kwargs) -> _R: ...

def _compare_digest(a: AnyStr, b: AnyStr, /) -> bool: ...



from _typeshed import ReadableBuffer
from collections.abc import Callable
from hashlib import _Hash

from paramiko.kex_group1 import KexGroup1 as KexGroup1

class KexGroup14(KexGroup1):
    P: int
    G: int
    name: str
    hash_algo: Callable[[ReadableBuffer], _Hash]

class KexGroup14SHA256(KexGroup14):
    name: str
    hash_algo: Callable[[ReadableBuffer], _Hash]



from _typeshed import Incomplete
from typing import Final

class Http:
    class ContentType:
        Xml: Final = "application/xml"
        Multipart: Final = "multipart/form-data"
        Json: Final = "application/json"

    @staticmethod
    def is_error_status(status): ...
    @staticmethod
    def raise_exception_from_status(status, message: Incomplete | None = None) -> None: ...
    config: Incomplete
    environment: Incomplete
    def __init__(self, config, environment: Incomplete | None = None) -> None: ...
    def post(self, path, params: Incomplete | None = None): ...
    def delete(self, path): ...
    def get(self, path): ...
    def put(self, path, params: Incomplete | None = None): ...
    def post_multipart(self, path, files, params: Incomplete | None = None): ...
    def http_do(self, http_verb, path, headers, request_body): ...
    def handle_exception(self, exception) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar, Literal

from .JpegImagePlugin import JpegImageFile

class MpoImageFile(JpegImageFile):
    format: ClassVar[Literal["MPO"]]
    def load_seek(self, pos) -> None: ...
    fp: Incomplete
    offset: Incomplete
    tile: Incomplete
    def seek(self, frame) -> None: ...
    def tell(self): ...
    @staticmethod
    def adopt(jpeg_instance, mpheader: Incomplete | None = None): ...



from _typeshed import Incomplete

class PatchRetentionRule:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self, type: str = "expire", every_seconds: int = 2592000, shard_group_duration_seconds: int | None = None
    ) -> None: ...
    @property
    def type(self): ...
    @type.setter
    def type(self, type) -> None: ...
    every_seconds: int
    @property
    def shard_group_duration_seconds(self): ...
    @shard_group_duration_seconds.setter
    def shard_group_duration_seconds(self, shard_group_duration_seconds) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete

class HDC:
    dc: Incomplete
    def __init__(self, dc) -> None: ...
    def __int__(self) -> int: ...

class HWND:
    wnd: Incomplete
    def __init__(self, wnd) -> None: ...
    def __int__(self) -> int: ...

class Dib:
    image: Incomplete
    mode: Incomplete
    size: Incomplete
    def __init__(self, image, size: Incomplete | None = None) -> None: ...
    def expose(self, handle): ...
    def draw(self, handle, dst, src: Incomplete | None = None): ...
    def query_palette(self, handle): ...
    def paste(self, im, box: Incomplete | None = None) -> None: ...
    def frombytes(self, buffer): ...
    def tobytes(self): ...

class Window:
    hwnd: Incomplete
    def __init__(self, title: str = "PIL", width: Incomplete | None = None, height: Incomplete | None = None) -> None: ...
    def ui_handle_clear(self, dc, x0, y0, x1, y1) -> None: ...
    def ui_handle_damage(self, x0, y0, x1, y1) -> None: ...
    def ui_handle_destroy(self) -> None: ...
    def ui_handle_repair(self, dc, x0, y0, x1, y1) -> None: ...
    def ui_handle_resize(self, width, height) -> None: ...
    def mainloop(self) -> None: ...

class ImageWindow(Window):
    image: Incomplete
    def __init__(self, image, title: str = "PIL") -> None: ...
    def ui_handle_repair(self, dc, x0, y0, x1, y1) -> None: ...



from _typeshed import Incomplete

class SourceLinks:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        _self: Incomplete | None = None,
        query: Incomplete | None = None,
        health: Incomplete | None = None,
        buckets: Incomplete | None = None,
    ) -> None: ...
    @property
    def query(self): ...
    @query.setter
    def query(self, query) -> None: ...
    @property
    def health(self): ...
    @health.setter
    def health(self, health) -> None: ...
    @property
    def buckets(self): ...
    @buckets.setter
    def buckets(self, buckets) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete
from typing import NoReturn

from .core import ACLCommands, DataAccessCommands, ManagementCommands, PubSubCommands, _StrType

class ClusterMultiKeyCommands:
    def mget_nonatomic(self, keys, *args): ...
    def mset_nonatomic(self, mapping): ...
    def exists(self, *keys): ...
    def delete(self, *keys): ...
    def touch(self, *keys): ...
    def unlink(self, *keys): ...

class ClusterManagementCommands(ManagementCommands):
    def slaveof(self, *args, **kwargs) -> None: ...
    def replicaof(self, *args, **kwargs) -> None: ...
    def swapdb(self, *args, **kwargs) -> None: ...

class ClusterDataAccessCommands(DataAccessCommands[_StrType]):
    def stralgo(
        self,
        algo,
        value1,
        value2,
        specific_argument: str = "strings",
        len: bool = False,
        idx: bool = False,
        minmatchlen: Incomplete | None = None,
        withmatchlen: bool = False,
        **kwargs,
    ): ...

class RedisClusterCommands(
    ClusterMultiKeyCommands, ClusterManagementCommands, ACLCommands[_StrType], PubSubCommands, ClusterDataAccessCommands[_StrType]
):
    def cluster_addslots(self, target_node, *slots): ...
    def cluster_countkeysinslot(self, slot_id): ...
    def cluster_count_failure_report(self, node_id): ...
    def cluster_delslots(self, *slots): ...
    def cluster_failover(self, target_node, option: Incomplete | None = None): ...
    def cluster_info(self, target_nodes: Incomplete | None = None): ...
    def cluster_keyslot(self, key): ...
    def cluster_meet(self, host, port, target_nodes: Incomplete | None = None): ...
    def cluster_nodes(self): ...
    def cluster_replicate(self, target_nodes, node_id): ...
    def cluster_reset(self, soft: bool = True, target_nodes: Incomplete | None = None): ...
    def cluster_save_config(self, target_nodes: Incomplete | None = None): ...
    def cluster_get_keys_in_slot(self, slot, num_keys): ...
    def cluster_set_config_epoch(self, epoch, target_nodes: Incomplete | None = None): ...
    def cluster_setslot(self, target_node, node_id, slot_id, state): ...
    def cluster_setslot_stable(self, slot_id): ...
    def cluster_replicas(self, node_id, target_nodes: Incomplete | None = None): ...
    def cluster_slots(self, target_nodes: Incomplete | None = None): ...
    def cluster_myshardid(self, target_nodes: Incomplete | None = None): ...
    def cluster_links(self, target_node: Incomplete): ...
    def cluster_flushslots(self, target_nodes: Incomplete | None = None) -> NoReturn: ...
    def cluster_bumpepoch(self, target_nodes: Incomplete | None = None) -> NoReturn: ...
    read_from_replicas: bool
    def readonly(self, target_nodes: Incomplete | None = None): ...
    def readwrite(self, target_nodes: Incomplete | None = None): ...
